@@ -2,6 +2,11 @@
 #ifndef liqcell_EASYRUN_H
 #define liqcell_EASYRUN_H
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+
 #include "liqbase.h"
 
 #include "liqcell.h"
@@ -115,7 +120,9 @@ typedef struct liqcellmouseeventargs
 	int         multiy;
 	int         multiw;
 	int         multih;
-
+	
+	int         multifx;		// first initial report obtained for the 2nd finger
+	int         multify;		// important to know we are now larger than we were at the start
 
 }
 	liqcellmouseeventargs;
@@ -123,6 +130,15 @@ typedef struct liqcellmouseeventargs
 
 liqcell * liqcell_easyrun_getactivecontrol();
 
+int liqcell_easyrun_mouseeventargs_multitouchprepare(liqcell *self, liqcellmouseeventargs *args,liqcell *context);
 
+extern int liqcell_easyrun_cursor_on_screen;
+extern int liqcell_easyrun_cursorflashcount;
+extern int liqcell_easyrun_fingerpressed;
+
+
+#ifdef __cplusplus
+}
+#endif
 
 #endif
