@@ -17,16 +17,18 @@
 #include <sys/time.h>		// req for getticks
 #include "liqapp.h"
 
-
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 //###################### not used yet, its still directly inside liqcell_easypaint
 
 
 
-int liqapp_url_wget(char *url,char *resultfilename,int resultbufsize)
+int liqapp_url_wget(const char *url, char *resultfilename, int resultbufsize)
 {
 
-	char *fn = url;
+	const char *fn = url;
 	
 
 			char cachefn[2048];
@@ -132,3 +134,8 @@ int liqapp_url_wget(char *url,char *resultfilename,int resultbufsize)
 			
 			return 0;
 }
+
+#ifdef __cplusplus
+}
+#endif
+
