@@ -22,11 +22,14 @@
  *
  */
 
+// if liqbase is run in wallmount mode, make sure this is set
+//#define LIQBASE_WALLMOUNT
+
 
 // 20090728_001621 lcuk : set this to have a 25fps limit to framerate, otherwise runs at fastest possible
 //#define LIMIT_FRAMERATE 1
 
-
+#define ABS(x)  ((x)<0 ? -(x) : (x))
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -37,7 +40,7 @@
 #include <sys/stat.h>
 #include <errno.h>
 #include <dirent.h>
-
+#include <pthread.h>
 
 #include "liqbase.h"
 
@@ -45,12 +48,16 @@
 #include "liqcell_prop.h"
 #include "liqcell_easyrun.h"
 #include "liqcell_easypaint.h"
+#include "liqapp_prefs.h"
+#include "liqaccel.h"
 
 #include "liqcell_historystore.h"
 
 #include "vgraph.h"
 
-
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 //extern liqcell *universe;
 
@@ -78,9 +85,8 @@ void liqapp_sleep_real(int millseconds)
 extern int liqcell_showdebugboxes;
 
 
-liqcell * liqcell_easyrun_currentdialog=NULL;;
-
-int liqcell_easyrun_activecontrol=NULL;
+liqcell * liqcell_easyrun_currentdialog=NULL;
+liqcell * liqcell_easyrun_activecontrol=NULL;
 int liqcell_easyrun_fingerpressed=0;
 
 
@@ -187,7 +193,9 @@ static void liqcellmouseeventargs_stroke_start(liqcellmouseeventargs *self,int m
 	self->multiy=0;
 	self->multiw=0;
 	self->multih=0;
-
+	
+	self->ox=0;
+	self->oy=0;
 
 	liqstroke_clear(self->stroke);
 	liqstroke_start(self->stroke,mx,my,mz,mt);
@@ -213,6 +221,9 @@ static void liqcellmouseeventargs_stroke_extend(liqcellmouseeventargs *self,int
 	self->mez=mz;
 	self->met=mt;
 
+	self->ox=0;
+	self->oy=0;
+
 	liqstroke_extend(self->stroke,mx,my,mz,mt);
 
 }
@@ -337,32 +348,34 @@ static void savethumb(liqcell *cell)
 	liqapp_ensurecleanusername(cellname);
 	
 	liqapp_log("...creating image %s",cellname);
-	liqimage *img = liqimage_newatsize(canvas.pixelwidth,canvas.pixelheight,0);
+	
+	// grab total live image?.. cant remember why i didnt do this normally?
+	//liqimage *img = canvas.rotation_native_surface;
+	liqimage *img = canvas.surface;
+	
+	//if(canvas.rotation_angle==0)
+	//	img = liqimage_newatsize(canvas.pixelwidth,canvas.pixelheight,0);
+	//else
+	//	img = liqimage_newatsize(canvas.pixelheight,canvas.pixelwidth,0);
 	
 	liqapp_log("...creating cliprect");
-	
 	liqcliprect *cr = liqcliprect_newfromimage(img);
-	
 	liqapp_log("...painting cell %s",cellname);
-	liqcell_easypaint(cell,cr,0,0,canvas.pixelwidth,canvas.pixelheight);
-	
+	//if(canvas.rotation_angle==0)
+	//	liqcell_easypaint(cell,cr,0,0,canvas.pixelwidth,canvas.pixelheight);
+	//else
+	//	liqcell_easypaint(cell,cr,0,0,canvas.pixelheight,canvas.pixelwidth);
 	liqapp_log("...building filename");
-
 				char 		fmtnow[255];
 	 			liqapp_formatnow(fmtnow,255,"yyyymmdd_hhmmss");
 				char buf[FILENAME_MAX+1];
 				snprintf(buf,FILENAME_MAX,"%s/sketches/liq.%s.%s.scr.png",app.userdatapath,fmtnow,cellname  );
-
-
-
-
-	
 	liqapp_log("...saving image as '%s'",buf);
 
 				liqimage_pagesavepng(img,buf);
                 
                 
-                void post_to_liqbase_net(char *filename,char *datakey);
+                void post_to_liqbase_net(const char *filename, const char *datakey);
                 
                 post_to_liqbase_net(buf,"screenshot");
 
@@ -384,8 +397,8 @@ static void savethumb(liqcell *cell)
 	liqapp_log("...releasing cr");
 	liqcliprect_release(cr);
 	
-	liqapp_log("...releasing image");
-	liqimage_release(img);
+	//liqapp_log("...releasing image");
+	//liqimage_release(img);
 	
 	liqapp_log("...done");
 	
@@ -403,19 +416,6 @@ static void savethumb(liqcell *cell)
 		return 1;
 	}
  */
-	static int tool_help_click(liqcell *self, liqcellclickeventargs *args, liqcell *tool)
-	{
-		return 1;
-	}
-	static int tool_bug_click(liqcell *self, liqcellclickeventargs *args, liqcell *tool)
-	{
-		return 1;
-	}
-	static int tool_tag_click(liqcell *self, liqcellclickeventargs *args, liqcell *tool)
-	{
-		return 1;
-	}
-	
 	static int tool_pic_click(liqcell *self, liqcellclickeventargs *args, liqcell *tool)
 	{
 		liqcell * content = liqcell_child_lookup(tool,"content");
@@ -425,12 +425,6 @@ static void savethumb(liqcell *cell)
 		return 1;
 	}
 	
-	static int tool_pin_click(liqcell *self, liqcellclickeventargs *args, liqcell *tool)
-	{
-		return 1;
-	}
-
-
 liqcell * toolclick(liqcell *vis)
 {
 	
@@ -471,48 +465,10 @@ liqcell * toolclick(liqcell *vis)
 		liqcell *b;
 		
 		int hh = 480-64;
-/*
-		b = liqcell_quickcreatevis("help","button",  800-50,64+hh*0.0,   50,+hh*0.2 );
-		liqcell_setfont(   b, liqfont_cache_getttf("/usr/share/fonts/nokia/nosnb.ttf", (24), 0) );
-		liqcell_handleradd_withcontext(b,    "click",   tool_help_click, self);
-		liqcell_propsets(  b,    "backcolor", "rgb(0,100,0)" );
-		liqcell_child_append( self, b );
-
-
-
-		b = liqcell_quickcreatevis("draw","button",  800-50,64+hh*0.2,   50,hh*0.2 );
-		liqcell_setfont(   b, liqfont_cache_getttf("/usr/share/fonts/nokia/nosnb.ttf", (24), 0) );
-		liqcell_handleradd_withcontext(b,    "click",   tool_bug_click, self);
-		liqcell_propsets(  b,    "backcolor", "rgb(100,0,0)" );
-		liqcell_child_append( self, b );
-	
-		
-		b = liqcell_quickcreatevis("tag","button",  800-50,64+hh*0.4,   50,hh*0.2);
-		liqcell_setfont(   b, liqfont_cache_getttf("/usr/share/fonts/nokia/nosnb.ttf", (24), 0) );
-		liqcell_handleradd_withcontext(b,    "click",   tool_tag_click, self);
-		liqcell_propsets(  b,    "backcolor", "rgb(0,100,100)" );
-		liqcell_child_append( self, b );		
-
-*/
-
-
-
-
-
-/*
-
-		b = liqcell_quickcreatevis("save","button",  800-50,64+hh*0.8,   50,hh*0.2 );
-		liqcell_setfont(   b, liqfont_cache_getttf("/usr/share/fonts/nokia/nosnb.ttf", (24), 0) );
-		liqcell_handleradd_withcontext(b,    "click",   tool_pin_click, self);
-		liqcell_propsets(  b,    "backcolor", "rgb(0,0,100)" );
-		liqcell_child_append( self, b );
-
-*/
-
 
 		b = liqcell_quickcreatevis("pic","button",  800-50,64+hh*0.8,   50,hh*0.2);
 		liqcell_setfont(   b, liqfont_cache_getttf("/usr/share/fonts/nokia/nosnb.ttf", (24), 0) );
-		liqcell_handleradd_withcontext(b,    "click",   tool_pic_click, self);
+		liqcell_handleradd_withcontext(b,    "click",   (void *)tool_pic_click, self);
 		liqcell_propsets(  b,    "backcolor", "rgb(100,100,0)" );
 		liqcell_child_append( self, b );
 
@@ -587,13 +543,13 @@ liqcell * toolclick(liqcell *vis)
 
 
 
-liqcell *liqcell_findfirsthandler(liqcell*root,char *handlername);
+liqcell *liqcell_findfirsthandler(liqcell*root, const char *handlername);
 
 
 
 
 
-liqcell *liqcell_findnexthandler(liqcell*self, liqcell*root,char *handlername)
+liqcell *liqcell_findnexthandler(liqcell*self, liqcell*root, const char *handlername)
 {
 	// 20090520_011358 lcuk : todo, fix this
 	// 20090812_003812 lcuk : simplified to use _visual branch walking
@@ -672,7 +628,7 @@ nextpar:
 		return NULL;
 }
 
-liqcell *liqcell_findfirsthandler(liqcell*root,char *handlername)
+liqcell *liqcell_findfirsthandler(liqcell*root, const char *handlername)
 {
 		if(liqcell_handlerfind(root,handlername))
 		{
@@ -726,6 +682,72 @@ int liqcell_easyrun_depth=0;
 int liqcell_easyrun_hide_tools = 0;
 int liqcell_easyrun_hide_back = 0;
 
+
+static int idle_lazyrun_shown(liqcell *start)
+{
+	if(start->kineticx || start->kineticy) return 1;
+	if(liqcell_getshown(start)==0)
+	{
+		// not yet shown!
+		//liqapp_log("idle_lazyrun_shown showing: '%s' (%d,%d)",start->name,start->x,start->y);
+		liqcell_handlerrun(start,"shown",NULL);
+		liqcell_setshown(start,1);
+		// also do some trickery for the image :)
+		if( liqcell_getimage(start)==NULL && liqcell_propgets(start,"imagefilename",NULL) != NULL)
+		{
+			liqcell_threadloadimage(start);
+		}
+		return 1;
+	}
+	liqcell *c=liqcell_getlinkchild_visual(start);
+	while(c)
+	{
+		if(idle_lazyrun_shown(c)) return 1;
+		c=liqcell_getlinknext_visual(c);
+	}
+	c=liqcell_getcontent(start);
+	if(c) return idle_lazyrun_shown(c);
+	return 0;
+}
+
+static liqcliprect *easyrun_realtime_reshape(liqcell *self, vgraph *graph)
+{
+	// if we are set to autorotate, stretch!
+	
+	if(canvas.fullscreen && (canvas.rotation_angle==0 || liqapp_pref_checkexists("forcerotationaspect") ))
+		vgraph_setscaleaspectlock(graph,  1);
+	else
+		vgraph_setscaleaspectlock(graph,  0);
+	//vgraph_setscaleaspectlock(graph,  0);
+	
+	vgraph_setcliprect(graph, NULL );
+	vgraph_settarget(graph,   NULL  );
+	
+	vgraph_setcliprect(graph, liqcliprect_hold(liqcanvas_getcliprect()) );
+	vgraph_settarget(graph,   liqimage_hold(liqcanvas_getsurface())  );
+	vgraph_setwindow( graph,  (self)                  );
+	
+	
+	//targetsurface = liqcanvas_getsurface();
+	return vgraph_getcliprect(graph);
+}
+
+static int slidetowards(int valuenow,int target)
+{
+	if(valuenow==target) return 0;
+	int diff = target - valuenow;
+	if(ABS(diff)>1)
+	{
+		if(ABS(diff)>3)
+			return diff/3 ;
+		else
+			return (diff<0)?-1:1;
+	}
+	else
+		return diff;
+}
+
+
 /**
  * Main program event handler.
  * @param self The liqcell to run
@@ -743,6 +765,9 @@ int liqcell_easyrun(liqcell *self)
 		return -1;
 	}
 	
+	// setup an auto rotation flag 
+	int autorotate = self->h > self->w;
+	
 	
 	if(liqcell_easyrunstack_used >= liqcell_easyrunstack_total)
 	{
@@ -788,56 +813,13 @@ int liqcell_easyrun(liqcell *self)
 	//vgraph_setwindow( graph,  (self)                  );
 	
 	
-liqimage    *targetsurface = NULL;//liqcanvas_getsurface();
+//liqimage    *targetsurface = NULL;//liqcanvas_getsurface();
 liqcliprect *targetcr      = NULL;//liqcanvas_getcliprect();
 
 int idle_lazyrun_wanted = liqcell_propgeti(self,"idle_lazyrun_wanted",0);
 
-	int idle_lazyrun_shown(liqcell *start)
-	{
-		if(start->kineticx || start->kineticy) return 1;
-		if(liqcell_getshown(start)==0)
-		{
-			// not yet shown!
-			liqcell_handlerrun(start,"shown",NULL);
-			liqcell_setshown(start,1);
-			return 1;
-		}
-		liqcell *c=liqcell_getlinkchild_visual(start);
-		while(c)
-		{
-			if(idle_lazyrun_shown(c)) return 1;
-			c=liqcell_getlinknext_visual(c);
-		}
-		c=liqcell_getcontent(start);
-		if(c) return idle_lazyrun_shown(c);
-		return 0;
-	}
 
-
-
-	void easyrun_realtime_reshape()
-	{
-		if(canvas.fullscreen)
-			vgraph_setscaleaspectlock(graph,  1);
-		else
-			vgraph_setscaleaspectlock(graph,  0);
-		//vgraph_setscaleaspectlock(graph,  0);
-		
-		vgraph_setcliprect(graph, NULL );
-		vgraph_settarget(graph,   NULL  );
-		
-		vgraph_setcliprect(graph, liqcliprect_hold(liqcanvas_getcliprect()) );
-		vgraph_settarget(graph,   liqimage_hold(liqcanvas_getsurface())  );
-		vgraph_setwindow( graph,  (self)                  );
-		
-		
-		targetsurface = liqcanvas_getsurface();
-		targetcr      = liqcanvas_getcliprect();
-		
-	}
-
-	easyrun_realtime_reshape();
+	targetcr = easyrun_realtime_reshape(self, graph);
 
 
 
@@ -870,7 +852,7 @@ int idle_lazyrun_wanted = liqcell_propgeti(self,"idle_lazyrun_wanted",0);
 	
 	if( (liqcell_easyrunstack_used==1) && (liqapp_pref_checkexists("noflashingcursor")==0) )
 	{
-		int tres=pthread_create(&cursorflashingthread,NULL,liqcell_easyrun_cursorflashingthread_function,self);
+		pthread_create(&cursorflashingthread,NULL,liqcell_easyrun_cursorflashingthread_function,self);
 	}
 
 
@@ -894,13 +876,6 @@ int 			hadmouse=0;
 int 			refreshinprogress=0;
 unsigned long 	refreshstarttime=0;		// if we have a refresh in progress
 
-
-
-			unsigned long 	ft1=tz0;
-			unsigned long 	ft2=tz0;
-
-
-
 liqcellpainteventargs paintargs;
 	//paintargs.cr = liqcanvas_getcliprect();
 	paintargs.graph = graph;
@@ -931,7 +906,64 @@ liqcellclickeventargs clickargs;
 	liqcell_easyrunstack[liqcell_easyrunstack_used].keyargs = &keyargs;
 	liqcell_easyrunstack[liqcell_easyrunstack_used].clickargs = &clickargs;
 	
+
+int fademode = 0;
+int fadestartx = 0;
+int fadestarty = 0;
+int fadeatx = 0;
+int fadeaty = 0;
+float fadeupto = 0.0;
+
+
+
+int slidemode = 0;		// slide is set to 0 == not moving, -1 == moving to previous, 1 == moving to next
+int slideuptox = 0;
+int slidestartx = 0;
+
+
 	
+	if( (liqcell_propgeti(self,"dialog_zoomed",0))  || (liqcell_easyrunstack_used==0))
+	{
+		// crap already hacked zoomin
+		// there shouldnt be any slidefade cos its already done earlier
+	}
+	else
+	{
+		// ooh nice, lets try it
+		
+		fadestartx = liqcell_propgeti(self,"fadestartx",0);
+		fadestarty = liqcell_propgeti(self,"fadestarty",0);
+		if(fadestartx || fadestarty)
+		{
+			// specified fadeslide in
+			fademode = 1;		// zooming in
+			fadeatx = fadestartx;
+			fadeaty = fadestarty;
+			fadeupto=1;
+		}
+		else
+		{
+			// NO specified fadeslide in, lets try this..
+			liqapp_log("liqcell_easyrun auto fade set to slidein");
+			fademode=1;
+			fadestartx = liqcliprect_getw(targetcr);
+			fadestarty = 0;
+
+			//fadestartx = 0;//liqcliprect_getw(targetcr);
+			//fadestarty = liqcliprect_getw(targetcr);
+
+			fadeatx = fadestartx;
+			fadeaty = fadestarty;
+			
+			fadeupto=1;
+			
+			
+		}
+	}
+	
+	
+
+
 
 
 int wx=0;
@@ -1108,7 +1140,12 @@ waitevent:
 			if( (ev.type == LIQEVENT_TYPE_KEY) && (ev.state==LIQEVENT_STATE_PRESS) && (ev.key.keycode==65307) )	//ESC
 			{
 				liqapp_log("Escape Pressed, Cancelling");
-				running=0;
+				//running=0;
+				if(fadestartx || fadestarty)
+				{	fademode=-1;  fadeatx=0; fadeaty=0; }
+				else
+				{	running=0; }
+
 				//result=NULL;
 				break;
 			}
@@ -1219,7 +1256,7 @@ waitevent:
 			}
 
 
-			else if(ev.type == LIQEVENT_TYPE_MOUSE && (zoom_in_progress==0)   )// && ev.mouse.pressure==0)
+			else if(ev.type == LIQEVENT_TYPE_MOUSE && (zoom_in_progress==0)  && (fademode==0)  )// && ev.mouse.pressure==0)
 			{
 				// mouse moving! w00t
 				
@@ -1229,7 +1266,10 @@ waitevent:
 				// get hold of actual coordinates...
 				int mx=ev.mouse.x;
 				int my=ev.mouse.y;
-			//	liqapp_log("event mouse scrn (%i,%i)",mx,my);
+				liqapp_log("event mouse scrn (%i,%i)",mx,my);
+			
+				mx -= fadeatx;	// let this have its position adjusted
+				my -= fadeaty;
 
 
 				// convert to be in the context of a cell
@@ -1237,7 +1277,8 @@ waitevent:
 				wy=0;
 
 				vgraph_convert_target2window(graph ,mx,my,  &wx,&wy);
-				//liqapp_log("mouse scrn (%i,%i)   cell (%i,%i)",mx,my,  wx,wy);
+				liqapp_log("mouse scrn (%i,%i)   cell (%i,%i)",mx,my,  wx,wy);
+				
 
 				hotx=0;
 				hoty=0;
@@ -1319,8 +1360,7 @@ waitevent:
 
 				}
 
-
-							//if( ((targetsurface->width-omsx)<64) && (omsy<64) && ((targetsurface->width-omex)<64) && (omey<64) && (liqcell_easyrun_hide_tools==0) )
+#ifdef LIQBASE_WALLMOUNT
 							if( ((canvas.pixelwidth-omsx)<64) && (omsy<64) && ((canvas.pixelwidth-omex)<64) && (omey<64) && (liqcell_easyrun_hide_tools==0) )
 							{
 								if(ev.mouse.pressure!=0)
@@ -1329,42 +1369,38 @@ waitevent:
 								}
 								else
 								{
-									
 									// tools button :)
-									
-
-									
 									if(strcasecmp( liqcell_getname(self), "tools" )==0)
 									{
 										// 20090606_013753 lcuk : if we are already in a tools object, close it again :)
-										running=0;
+										//running=0;
+										if(fadestartx || fadestarty)
+										{	fademode=-1;  fadeatx=0; fadeaty=0; }
+										else
+										{	running=0; }
 										goto quickfin;
 										
 									}
-
 											// 20090606_003219 lcuk : mark these as clear and shortcut the process
 											mouseargs.mcnt=0;
 											mouseargs.hit = NULL;
-
 											hot=self;
 											hotx=0;
 											hoty=0;
-
 											zoom_app=toolclick(self);
 											zoom_in_progress=1;
 											zoom_start=liqapp_GetTicks();
 											zoom_direction = 1;
-
 									goto quickfin;
-
-
 								}
-								
 							}
-						
+#endif						
 
-							//if( (omsx<64) && ((targetsurface->height-omsy)<64) && (omex<64) && ((targetsurface->height-omey)<64) )
-							if( (omsx<64) && ((canvas.pixelheight-omsy)<64) && (omex<64) && ((canvas.pixelheight-omey)<64) && (liqcell_easyrun_hide_back==0)  )
+
+
+
+
+							if(  ( ((omsx)<80) && (omsy<56) && ((omex)<80) && (omey<56) ) && ( liqcell_easyrun_hide_back==0) )
 							{
 								if(ev.mouse.pressure!=0)
 								{
@@ -1373,8 +1409,12 @@ waitevent:
 								else
 								{
                                     hot=NULL;
-									running=0;
-									goto quickfin;
+									// todo, call the shell command to slide back to dashboard (cant find the page right now)
+									
+									// monimize before actually going and hopefully we will not flash green ?!
+									liqcanvas_minimize();
+
+									system("/usr/bin/dbus-send --type=signal --session /com/nokia/hildon_desktop com.nokia.hildon_desktop.exit_app_view");
 								}
 								
 							}
@@ -1384,6 +1424,63 @@ waitevent:
 
 
 
+
+
+
+#ifdef LIQBASE_WALLMOUNT
+
+							if( (omsx<64) && ((canvas.pixelheight-omsy)<64) && (omex<64) && ((canvas.pixelheight-omey)<64) && (liqcell_easyrun_hide_back==0)  )
+#else
+							//ROTATEPATCHPOINT replace with commented out block
+							if( ((canvas.pixelwidth-omsx)<80) && (omsy<56) && ((canvas.pixelwidth-omex)<80) && (omey<56) && (liqcell_easyrun_hide_back==0) )
+							//if( ((canvas.pixelwidth-omsx)<80) && (canvas.pixelheight-omsy<56) && ((canvas.pixelwidth-omex)<80) && (canvas.pixelheight-omey<56) && (liqcell_easyrun_hide_back==0) )
+
+#endif
+							{
+								if(ev.mouse.pressure!=0)
+								{
+									hot=NULL;
+								}
+								else
+								{
+                                    hot=NULL;
+									//running=0;
+									if(fadestartx || fadestarty)
+									{	fademode=-1;  fadeatx=0; fadeaty=0; }
+									else
+									{	running=0; }
+									goto quickfin;
+								}
+								
+							}
+
+
+// for future reference
+// the act of painting a cell makes it wet.
+// this state is the highest memory usage.
+// the cell should retain this value
+// and if skipped on subsequent renders shall be checked
+// and if it is no longer visible, it dries out.
+// a dried event should occur.
+// note, this should be most effective within lists
+// to give the individual tiles a chance to reevaluate themselves
+// i suppose writing the framenumber to a property of cell would suffice
+// then in the render loop if rendering fails due to being outside and the index was set previously
+// send the event and clear the flag
+// any cell listening for this gets a chance to release its data
+
+// adjust and tweak, maybe allow it to skip 5 frames or something
+// it might be nice to still allow a fair scrollback
+// worth a try :)
+// the cell might choose to reset its shown flag too :)
+// which would make the entire process nicely reusable
+// even for more than simple widgets
+
+
+							mouseargs.ox=0;
+							mouseargs.oy=0;
+	
+
 				if(hot)// && hot==mouseargs.hit)
 				{
 					// still using the right area :)
@@ -1414,6 +1511,7 @@ waitevent:
 							liqcell *ohit=vhit;
 							while(ohit && ohit!=self)
 							{
+								//liqapp_log("huh easyrun (%d,%d) += (%d,%d) '%s'",ox,oy,ohit->x,ohit->y,ohit->name);
 								ox+=ohit->x;
 								oy+=ohit->y;
 								ohit=liqcell_getlinkparent(ohit);
@@ -1424,7 +1522,7 @@ waitevent:
 							
 							
 							liqcell_easyrun_mouseeventargs_multitouchprepare(vhit,&mouseargs,NULL);
-
+							
 							if( liqcell_handlerrun(vhit,"mouse",&mouseargs) )
 							{
 								// handled it \o/
@@ -1435,6 +1533,10 @@ waitevent:
 								}
 							}
 						}
+						else
+						{
+							// NO mouse event handler - we should now try to slide..
+						}
 					}
 
 
@@ -1465,6 +1567,10 @@ waitevent:
 								{
 									liqapp_log("click run '%s'",vhit->name);
 									if( liqcell_handlerrun(vhit,"click",&clickargs) )
+									{}
+									
+									
+									// dont care about result, we knew we had a click event, and we should do this anyway
 									{
                                         
                                         liqcanvas_settitle( liqcell_getcaption(self) );
@@ -1484,16 +1590,71 @@ waitevent:
 
 
 										wantwait=0;
-										//refreshinprogress=0;
+										refreshinprogress=0;		// bring this back too
 										dirty=1;
 										hadmouse=1;
+										liqapp_log("click done");
 
 									}
                                     
-                                    easyrun_realtime_reshape();
+                                    targetcr = easyrun_realtime_reshape(self, graph);
 								}
 							}
 						}
+
+
+
+
+
+						else
+						{
+							// NO click event handler
+							// this is essentially a free hit
+							// if i
+							int dx = mouseargs.stroke->pointlast->x - mouseargs.stroke->pointfirst->x;
+							int dy = mouseargs.stroke->pointlast->y - mouseargs.stroke->pointfirst->y;
+							int ar=0;
+							liqapp_log("ARGMOUSE xy(%d,%d)",dx,dy);
+							if(ABS(dx)>ABS(dy) ) // && ((ABS(dx)/4) > ABS(dy)))
+							{
+								
+								liqapp_log("ARGMOUSE2 xy(%d,%d)",dx,dy);
+								
+								liqcell *adj;
+								if(dx<0)
+								{
+									adj = liqcell_getlinkprev_visual(self);
+								}
+								else
+								{
+									adj = liqcell_getlinknext_visual(self);
+								}
+								if(adj)
+								{
+									liqapp_log("ARGMOUSE 2 xy(%d,%d) '%s'",dx,dy,adj->name);
+									// swap self
+									liqcell *orig=self;
+									if(liqcell_getselected(orig))
+									{
+										liqcell_setselected(self,0);
+										liqcell_setselected(adj,1);
+										
+									}
+									liqcell_ensurevisible_centred(adj);
+									self=adj;
+									dirty=1;
+									//targetcr = easyrun_realtime_reshape(self, graph);
+									
+								}
+							}
+							
+						}
+
+
+
+
+
+
 					}
 
 				}
@@ -1553,7 +1714,7 @@ quickfin:
 			{
 				liqapp_log("event resize");
 				
-				easyrun_realtime_reshape();
+				targetcr = easyrun_realtime_reshape(self, graph);
 				
 				refreshinprogress=0;
 				wantwait=0;
@@ -1580,6 +1741,7 @@ quickfin:
 
 			else
 			{
+				if(fademode) break;
 				// anything else, just ignore it
 //#ifndef USE_MAEMO
 				wantwait=1;
@@ -1591,15 +1753,78 @@ quickfin:
 //skipev:
 
 
-		if(refreshinprogress==0 && self->visible==0)break;
+		if(refreshinprogress==0 && self->visible==0) break;
 		
-		// check for dialog complete dignal and bail
+		// check for dialog complete signal and bail
 		
-		if(refreshinprogress==0 && liqcell_propgeti(self,"dialog_running",1)==0 ) break;
+		
+		
+		//if(refreshinprogress==0 && (fademode==0) && liqcell_propgeti(self,"dialog_running",1)==0 ) break;
+		
+		
+		
+		if( (refreshinprogress==0) && (fademode==0) && liqcell_propgeti(self,"dialog_running",1)==0 )
+		{
+			if(fadestartx || fadestarty)
+			{
+				// fade out!
+				fademode=-1;
+			}
+			else
+			{
+				// just bail, we are out of here
+				break;
+			}
+		}
 
 
 
-		float zoom_duration = 0.15;//0.4;//0.01;//0.1;//0.2;	// time to go from fullscreen to zoomed in
+		float zoom_duration = 0.2; //0.15;//0.4;//0.01;//0.1;//0.2;	// time to go from fullscreen to zoomed in
+		
+		// todo make this a system config option
+		
+		if((fademode) && (refreshinprogress==0))
+		{
+			//liqapp_log("liqcell_easyrun fade in %i,%i",fadeatx,fadeaty);
+			if(fademode==1)
+			{
+				// just halve distance per step
+				fademode=0;
+				int diffx = slidetowards(fadeatx,0);
+				int diffy = slidetowards(fadeaty,0);
+				
+				//liqapp_log("liqcell_easyrun fade in %i,%i, +1(a) diff %i,%i",fadeatx,fadeaty,diffx,diffy);
+				
+				if(diffx) { fadeatx+=diffx; dirty=1; fademode=1; } else { fadeatx = 0; }
+				if(diffy) { fadeaty+=diffy; dirty=1; fademode=1; } else { fadeaty = 0; }
+				dirty=1;
+				
+				fadeupto = (float)(fadeatx+fadeaty) / (float)(fadestartx+fadestarty);
+				
+				//liqapp_log("liqcell_easyrun fade in %i,%i, +1(b) diff %i,%i",fadeatx,fadeaty,diffx,diffy);
+			}
+			else
+			{
+				fademode=0;
+				int diffx = slidetowards(fadeatx,fadestartx);
+				int diffy = slidetowards(fadeaty,fadestarty);
+				
+				//liqapp_log("liqcell_easyrun fade in %i,%i, -1 diff %i,%i",fadeatx,fadeaty,diffx,diffy);
+				
+				if(diffx) { fadeatx+=diffx; dirty=1; fademode=-1; } else { fadeatx = fadestartx; }
+				if(diffy) { fadeaty+=diffy; dirty=1; fademode=-1; } else { fadeaty = fadestarty; }
+				dirty=1;
+				
+				fadeupto = (float)(fadeatx+fadeaty) / (float)(fadestartx+fadestarty);
+				
+				// if the fademode is now unset, we change fademode=0; and running=0;
+				//if(ABS(diffx)<5 && ABS(diffy)<5)running=0;
+				
+				if(fadeupto > 0.95)running=0;
+			}
+		}
+		//###########################################################
+		
 		if((zoom_in_progress) && (hot) && (refreshinprogress==0))
 		{
 			float zoomruntime = (liqapp_GetTicks()-zoom_start) / (1000.0);
@@ -1614,12 +1839,16 @@ quickfin:
 
 					//zoom_in_progress=0;
 					// run the zoom_app :)
+					
+					liqcell_propseti((liqcell *)zoom_app,"dialog_zoomed",1);
 
-					liqcell_easyrun( zoom_app );
+						liqcell_easyrun( (liqcell *)zoom_app );
+					
+					liqcell_propremovei((liqcell *)zoom_app,"dialog_zoomed");
                     
                     liqcanvas_settitle( liqcell_getcaption(self) );
                     
-                    easyrun_realtime_reshape();
+                    targetcr = easyrun_realtime_reshape(self, graph);
                     
                     // Mon Sep 07 22:13:27 2009 lcuk : was not releasing... tsk tsk
 // i bet this is where the bug is coming from.....
@@ -1675,6 +1904,7 @@ moar:
 
 			dirty=1;
 		}
+		
 		if(self->dirty && (refreshinprogress==0))// && (dirty==0))
 		{
 			
@@ -1691,12 +1921,37 @@ moar:
 		}
 		if(refreshinprogress==0) if(running==0) break;
 		if(paintargs.runfast) dirty=1;
+		
+		
+		 // remove autorotate from here, i should enable it per dialog
+		 // because by putting it here messes up normal stuff and thats not desired
+		 // put it back again (31/12)
+		 // changed it yet again to be a specific, per dialog flag
+		if( (dirty==0) && (refreshinprogress==0) && (mouseargs.mcnt==0) && (liqcell_propgeti(self,"autorotate",0)==1) )// liqapp_pref_checkexists("autorotate") )
+		{
+			int aax=0;
+			int aay=0;
+			int aaz=0;
+			
+			liqaccel_read(&aax,&aay,&aaz);
+			dirty=1;
+			
+		}
+		
+		
+		
 		if(((dirty==1) && (refreshinprogress==0)))
 		{
 		//	liqapp_log("render %i  ud=%i",framecount,universe->dirty);
 			//liqapp_log("rendering %i",framecount);
 			//liqcliprect_drawclear(liqcanvas_getcliprect(),255,128,128);
-			liqcliprect_drawclear(liqcanvas_getcliprect(),0,128,128);
+			
+			
+			//liqcliprect_drawclear(liqcanvas_getcliprect(),0,128,128);
+			vgraph_setbackcolor(graph, vcolor_YUV(0,128,128) );
+			vgraph_drawclear(graph);
+			
+			
 			// ensure runfast is unset before attmpting the next loop
 			paintargs.runfast=0;
 			//##################################################### render handler
@@ -1728,7 +1983,56 @@ moar:
 			//liqcell_easyrun_cursor_on_screen = 0;
 			
 			//liqapp_log("render drawing wh(%i,%i)",w,h);
+			
+			
+			
+			
+			
+			if(fademode)
+			{
+								
+				//liqapp_log("liqcell_easyrun fade in %i,%i, fm(%i) stack(%i)",fadeatx,fadeaty,fademode,liqcell_easyrunstack_used );
+
+				if(liqcell_easyrunstack_used>0)
+				{
+					// draw the background
+					liqcell *prev = liqcell_easyrunstack[liqcell_easyrunstack_used-1].runself;
+					//liqapp_log("liqcell_easyrun fade : drawing prev %s",prev->name);
+					
+					// optimize this soon to ONLY draw + darken the visible area :)
+					// adjust the cliprect to compensate for the covered area
+					// this should block the rest automatically, but th einterface isnt there yet
+					
+					unsigned char backbright = ((1.0-fadeupto) * 255.0);
+					
+					liqapp_log("fadeupto %3.3f   %i",fadeupto,backbright);
+					if(fadeatx>0 && fadeaty==0)
+					{
+						vgraph_drawcell(graph,x-(w-fadeatx),y,w,h,prev);
+						//vgraph_setbackcolor(graph, vcolor_YUVA(0,128,128, backbright ) );
+						//vgraph_drawrectfadeoutcolor(graph,x-(w-fadeatx),y,w,h);
+
+					}
+					else
+					{
+						vgraph_drawcell(graph,x,y,w,h,prev);
+						//vgraph_setbackcolor(graph, vcolor_YUVA(0,128,128, backbright) );
+						//vgraph_drawrectfadeoutcolor(graph,x,y,w,h);
+
+					}
+				}
+				// adjust offset
+				x+=fadeatx;
+				y+=fadeaty;
+				
+				// dark fill the area here
+				vgraph_setbackcolor(graph, vcolor_YUVA(0,128,128, 0) );
+				vgraph_drawrect(graph,x,y,w,h);
+			}
+
+			
 			vgraph_drawcell(graph,x,y,w,h,self);
+			
 			//liqapp_log("render adding nav items");
 			static liqimage *infoback=NULL;
 			static liqimage *infoclose=NULL;
@@ -1756,23 +2060,54 @@ moar:
                 
 			}
 
-            
+#ifdef LIQBASE_WALLMOUNT
+
 			if(infoback && infoclose &&     (liqcell_easyrun_hide_back==0)    )
 			{
 				//liqapp_log("************************************************************************************** use");
 				if(liqcell_easyrun_depth==1)
-					liqcliprect_drawimagecolor(targetcr, infoclose, 0,targetsurface->height-48,48,48, 1);
+					liqcliprect_drawimagecolor(targetcr, infoclose, 0,liqcliprect_getw(targetcr)-48,48,48, 1);
 				else
 				
-					liqcliprect_drawimagecolor(targetcr, infoback , 0,targetsurface->height-48,48,48, 1);
+					liqcliprect_drawimagecolor(targetcr, infoback , 0,liqcliprect_geth(targetcr)-48,48,48, 1);
 			}
             
             
 			if( (infotools) && (liqcell_easyrun_hide_tools==0) )
 			{
-					liqcliprect_drawimagecolor(targetcr, infotools , targetsurface->width-48,0 ,48,48, 1);
+					liqcliprect_drawimagecolor(targetcr, infotools , liqcliprect_getw(targetcr)-48,0 ,48,48, 1);
 			}
- 		
+
+#else
+
+			if(infoback && infoclose &&     (liqcell_easyrun_hide_back==0)    )
+			{
+				//liqapp_log("************************************************************************************** use");
+				
+				//liqapp_log("infoback %i,%i",infoback->width,infoback->height);
+				
+				//ROTATEPATCHPOINT - switch this if with the next commented out variation
+				if(liqcell_easyrun_depth==1)
+					liqcliprect_drawimagecolor(targetcr, infoclose, liqcliprect_getw(targetcr)-80,0 ,80,56, 1);
+				else		
+					liqcliprect_drawimagecolor(targetcr, infoback , liqcliprect_getw(targetcr)-80,0 ,80,56, 1);
+
+
+				//if(liqcell_easyrun_depth==1)
+				//	liqcliprect_drawimagecolor(targetcr, infoclose, liqcliprect_getw(targetcr)-80,liqcliprect_geth(targetcr)-56 ,80,56, 1);
+				//else		
+				//	liqcliprect_drawimagecolor(targetcr, infoback , liqcliprect_getw(targetcr)-80,liqcliprect_geth(targetcr)-56 ,80,56, 1);
+
+			}
+            
+            
+			//if( (infotools) && (liqcell_easyrun_hide_tools==0) )
+			//{
+			//		liqcliprect_drawimagecolor(targetcr, infotools , liqcliprect_getw(targetcr)-48,0 ,48,48, 1);
+			//}
+
+
+#endif
 			// 20090712_224448 lcuk : i want MAX 25fps, just to see
 			//ft2=liqapp_GetTicks();
 #ifdef LIMIT_FRAMERATE		
@@ -1786,29 +2121,12 @@ moar:
 #endif
 
 
-
-
-	static const char *cpufreq_filename = "/sys/devices/system/cpu/cpu0/cpufreq/cpuinfo_cur_freq";
-	//static const char *cpufreq_filename = "/sys/devices/system/cpu/cpu0/cpufreq/scaling_cur_freq";
-	int cpufreq_read(int *freq)
-	{
-		FILE *fd;
-		int rs;
-		*freq=0;
-		fd = fopen(cpufreq_filename, "r");
-		if(fd==NULL){ liqapp_log("cpufreq, cannot open for reading"); return -1;}	
-		rs=fscanf((FILE*) fd,"%i",freq);	
-		fclose(fd);	
-		if(rs != 1){ liqapp_log("cpufreq, cannot read information"); return -2;}
-	}
-
-
-
 			//liqapp_log("render adding framecount");		
 // 20090520_014021 lcuk : show frame information
 
 
-/*
+if(liqcell_showfps)
+{
 			static liqfont *infofont=NULL;
 			if(!infofont)
 			{
@@ -1816,21 +2134,20 @@ moar:
 			}
 			if(0==liqfont_setview(infofont, 1,1 ))
 			{
-				char *cap=liqcell_getcaption(self);
+				const char *cap=liqcell_getcaption(self);
 				if(!cap || !*cap) cap="[nameless]";
 				char buff[255];
-				int cpufreq=0;
-				//cpufreq_read(&cpufreq);
 				snprintf(buff,sizeof(buff),"%s '%s' %3i, %3.3f, %3.3f",app.title,cap,framecount, liqapp_fps(tz0,tz1,1) ,liqapp_fps(tzs,tz1,framecount) );
-				liqapp_log(buff);
+			//	liqapp_log(buff);
 				int hh=liqfont_textheight(infofont);
-				liqcliprect_drawtextinside_color(targetcr, infofont,  0,0, targetsurface->width,hh, buff,0, 255,128,128);
+				liqcliprect_drawtextinside_color(targetcr, infofont,  0,0, liqcliprect_getw(targetcr),hh, buff,0, 255,128,128);
 			}
- */ 
+} 
  		
 			//liqapp_log("render refreshing");
 			
 			
+			
 
 			
 			
@@ -1908,3 +2225,8 @@ moar:
 	
 	return result;
 }
+
+#ifdef __cplusplus
+}
+#endif
+
