@@ -4,13 +4,134 @@
 #include <stdio.h>
 
 #include <memory.h>
+#include <math.h>
 
 #include "liqcliprect.h"
 #include "liq_xsurface.h"
 #include "liqapp.h"
+#include "liqapp_prefs.h"
+#include "liqcell_easypaint.h"
 #include "liqcanvas.h"
 #include "liqimage.h"
 #include "liqsketch.h"
+#include "liqaccel.h"
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+//############################################### costable
+#define PI 3.14159265358979323846f
+
+#define costable_size      (512)
+#define costable_size_div2 (costable_size / 2)
+#define costable_size_div4 (costable_size / 4)
+#define costable_size_last (costable_size - 1)
+
+float   costable[ costable_size ];
+int     costable_fudged[ costable_size ];
+int     costable_ready = 0;
+#define costable_fudge 1024
+
+inline float costable_getcos(float n)
+{
+	float f = n * costable_size_div2 / PI;
+	int i = f;
+	if (i < 0) i=-i;
+	return costable[ (i + costable_size_div4) % costable_size_last];
+}
+
+inline float costable_getsin(float n)
+{
+	float f = n * costable_size_div2 / PI;
+	int i = f;
+	if (i < 0)
+		return costable[ -((-i) % costable_size_last) + costable_size  ];
+	else
+		return costable[ (i) % costable_size_last];
+}
+
+inline float costable_fudgegetcos(float n)
+{
+	float f = n * costable_size_div2 / PI;
+	int i = f;
+	if (i < 0) i=-i;
+	return costable_fudged[ (i + costable_size_div4) % costable_size_last];
+}
+
+inline float costable_fudgegetsin(float n)
+{
+	float f = n * costable_size_div2 / PI;
+	int i = f;
+	if (i < 0)
+		return costable_fudged[ -((-i) % costable_size_last) + costable_size  ];
+	else
+		return costable_fudged[ (i) % costable_size_last];
+}
+
+
+void costable_init()
+{
+	if(costable_ready)return;
+	int i;
+	for (i=0;i<costable_size;i++)
+	{
+		costable[i] = (float)sin(i * PI / costable_size_div2);
+		costable_fudged[i] = costable_fudge * sin(i * PI / costable_size_div2);
+	}
+	costable_ready=1;
+
+}
+
+//###############################################
+
+				// stupendously slow proof of concept
+
+
+				void matrot_slow( int cx,int cy, int *px,int *py,float angle )
+				{
+					if(!angle)return;	// all ok
+					float p1x = *px - cx;
+					float p1y = *py - cy;
+					// finally rotating
+					
+					*px = cx + p1x * cos(angle) - p1y * sin(angle);
+					*py = cy + p1x * sin(angle) + p1y * cos(angle);
+					
+				}
+				
+				// make it faster
+				// improvise however required :)
+				
+
+				void matrot_mid( int cx,int cy, int *px,int *py,float angle )
+				{
+					if(!angle)return;	// all ok
+					float p1x = *px - cx;
+					float p1y = *py - cy;
+					// finally rotating
+					
+					*px = cx + p1x * costable_getcos(angle) - p1y * costable_getsin(angle);
+					*py = cy + p1x * costable_getsin(angle) + p1y * costable_getcos(angle);
+					
+				}
+		
+		 		// maybe a bit buggy, testing previous version
+				// seems stable now :) and fast enough
+				// it was reading the accelerometer so often
+				
+				void matrot( int cx,int cy, int *px,int *py,float angle )
+				{
+					if(!angle)return;	// all ok
+					int p1x = *px - cx;
+					int p1y = *py - cy;
+					// finally rotating
+					
+					*px = cx + (p1x * costable_fudgegetcos(angle) / costable_fudge) - (p1y * costable_fudgegetsin(angle) / costable_fudge);
+					*py = cy + (p1x * costable_fudgegetsin(angle) / costable_fudge) + (p1y * costable_fudgegetcos(angle) / costable_fudge);
+					
+				}
+		
 //##################################################################
 //##################################################################
 //##################################################################
@@ -79,14 +200,14 @@ void liqcliprect_free(liqcliprect *self)
 //#########################################################################
 
 
-inline int liqcliprect_isvalid(liqcliprect *self)
+int liqcliprect_isvalid(liqcliprect *self)
 {
 	if(self->sx>=self->ex)return 0;
 	if(self->sy>=self->ey)return 0;
 	return 1;
 }
 
-inline void liqcliprect_shrink(liqcliprect *self,int sx,int sy,int ex,int ey)
+void liqcliprect_shrink(liqcliprect *self,int sx,int sy,int ex,int ey)
 {
 	// given a liqcliprect and another rectangle,
 	if(sx > self->sx) self->sx=sx;
@@ -96,7 +217,7 @@ inline void liqcliprect_shrink(liqcliprect *self,int sx,int sy,int ex,int ey)
 	if(self->ex<self->sx)self->ex=self->sx;
 	if(self->ey<self->sy)self->ey=self->sy;
 }
-inline void liqcliprect_copy(liqcliprect *self,liqcliprect *other)
+void liqcliprect_copy(liqcliprect *self,liqcliprect *other)
 {
 	self->sx=other->sx;
 	self->sy=other->sy;
@@ -163,7 +284,7 @@ int    	liqcliprect_geth(liqcliprect *self)
 
 
 
-inline void liqcliprect_drawclear(liqcliprect *self,unsigned char grey,unsigned char u,unsigned char v)
+void liqcliprect_drawclear(liqcliprect *self,unsigned char grey,unsigned char u,unsigned char v)
 {
 	// todo follow the clipregion rules tsk tsk
 	if(self->sx==0 && self->sy==0 && self->ex==(self->surface->width-1) && self->ey==(self->surface->height-1))
@@ -181,7 +302,7 @@ void 		liqcliprect_drawpsetcolor(			liqcliprect *self,int x, int y, unsigned cha
 }
 
 
-inline void liqcliprect_drawpgetcolor(      	liqcliprect *self,int x, int y, unsigned char *grey,unsigned char *u,unsigned char *v)
+void liqcliprect_drawpgetcolor(      	liqcliprect *self,int x, int y, unsigned char *grey,unsigned char *u,unsigned char *v)
 {
 	xsurface_drawpget_yuv(self->surface,x,y,grey,u,v);
 
@@ -249,13 +370,37 @@ void liqcliprect_drawlinecolor(liqcliprect *self,int x1, int y1, int x2, int y2,
 	xsurface_drawline_yuv(self->surface,x1,y1,x2,y2,grey,u,v);
 }
 
+void		liqcliprect_drawthicklinecolor(		liqcliprect *self,int x1, int y1,int x2, int y2, unsigned char thickness, unsigned char grey,unsigned char u,unsigned char v)
+{
+	if(thickness<=1)
+	{
+		
+		liqcliprect_drawlinecolor(self,x1,y1,x2,y2,grey,u,v);
+		return;
+	}
+	// full line x1..x2,y1..y2
+	if(x1<self->sx) return;
+	if(x1>self->ex) return;
+	if(y1<self->sy) return;
+	if(y1>self->ey) return;
+
+	if(x2<self->sx) return;
+	if(x2>self->ex) return;
+	if(y2<self->sy) return;
+	if(y2>self->ey) return;
+	// draw the line now cleanly and technically without any further bound checking - it is 100% inside the boundary
+	xsurface_drawthickline_yuv(self->surface,x1,y1,x2,y2,thickness,grey,u,v);
+}
+
 //##################################################################
 //##################################################################
 //##################################################################
 
 void liqcliprect_drawboxlinecolor(liqcliprect *self,int x,int y,int w,int h,unsigned char grey,unsigned char u,unsigned char v)
 {
-	// do not use yet, its not ready
+	
+//	if(x&1){x--;w++; }
+//	if((x+w)&1){ w++; }
 int r=x+w-1;
 int b=y+h-1;
 		liqcliprect_drawlinerowcolor(self,x,y,  r,y,grey,u,v);
@@ -375,6 +520,8 @@ int b=(y+h)-1;
 	if(b&1)b++;
     
     //xsurface_drawrect_grey(self->surface,x,y,w,h,grey);
+	
+
 
 	int tw=(r-x)/16;
 	int th=(b-y)/16;
@@ -450,7 +597,7 @@ int b=(y+h)-1;
 
 
 
-inline void liqcliprect_drawglyph_grey(liqcliprect *self,liqfont *font,int x,int y,unsigned char glyph)
+void liqcliprect_drawglyph_grey(liqcliprect *self,liqfont *font,int x,int y,unsigned char glyph)
 {
 liqfontglyph *g = liqfont_getglyph(font,glyph);
 	if(!g)return;
@@ -589,7 +736,7 @@ void liqcliprect_drawtextinside(liqcliprect *self,liqfont *font,int x,int y,int
 //########################################################################
 
 
-inline void liqcliprect_drawglyph_color(liqcliprect *self,liqfont *font,int x,int y,unsigned char glyph,unsigned char grey,unsigned char u,unsigned char v)
+void liqcliprect_drawglyph_color(liqcliprect *self,liqfont *font,int x,int y,unsigned char glyph,unsigned char grey,unsigned char u,unsigned char v)
 {
 	if((grey==255) && (u==128) && (v==128)){ liqcliprect_drawglyph_grey(self,font,x,y,glyph); return; }
 
@@ -657,28 +804,58 @@ liqfontglyph *g = liqfont_getglyph(font,glyph);
 
 int liqcliprect_drawtext_color(liqcliprect *self,liqfont *font,int xs,int ys,char *data,unsigned char grey,unsigned char u,unsigned char v)
 {
-	int x=xs;
-	unsigned char ch;
-	while ( (ch=*data++) )
+	if(font->rotation==0 || font->rotation==180)
 	{
-		liqcliprect_drawglyph_color(self,font,x,ys, ch , grey,u,v);
-		x+=liqfont_getglyphwidth(font,ch);//font->glyphwidths[ch];
+		int x=xs;
+		unsigned char ch;
+		while ( (ch=*data++) )
+		{
+			liqcliprect_drawglyph_color(self,font,x,ys, ch , grey,u,v);
+			x+=liqfont_getglyphwidth(font,ch);//font->glyphwidths[ch];
+		}
+		return x;
+	}
+	else
+	{
+		int y=ys;
+		unsigned char ch;
+		while ( (ch=*data++) )
+		{
+			liqcliprect_drawglyph_color(self,font,xs,y, ch , grey,u,v);
+			y+=liqfont_getglyphheight(font,ch);//font->glyphwidths[ch];
+		}
+		return y;
 	}
-	return x;
 }
 
 int liqcliprect_drawtextn_color(liqcliprect *self,liqfont *font,int xs,int ys,char *data,int datalen,unsigned char grey,unsigned char u,unsigned char v)
 {
-	int x=xs;
-	unsigned char ch;
-	if(datalen<=0)return x;
-	while(datalen--)
+	if(font->rotation==0 || font->rotation==180)
+	{	
+		int x=xs;
+		unsigned char ch;
+		if(datalen<=0)return x;
+		while(datalen--)
+		{
+			ch=*data++;
+			liqcliprect_drawglyph_color(self,font,x,ys, ch , grey,u,v );
+			x+=liqfont_getglyphwidth(font,ch);//font->glyphwidths[ch];
+		}
+		return x;
+	}
+	else
 	{
-		ch=*data++;
-		liqcliprect_drawglyph_color(self,font,x,ys, ch , grey,u,v );
-		x+=liqfont_getglyphwidth(font,ch);//font->glyphwidths[ch];
+		int y=ys;
+		unsigned char ch;
+		if(datalen<=0)return y;
+		while(datalen--)
+		{
+			ch=*data++;
+			liqcliprect_drawglyph_color(self,font,xs,y, ch , grey,u,v );
+			y+=liqfont_getglyphheight(font,ch);//font->glyphwidths[ch];
+		}
+		return y;
 	}
-	return x;
 }
 
 void liqcliprect_drawtextcentredon_color(liqcliprect *self,liqfont *font,int cx,int cy,char *text,unsigned char grey,unsigned char u,unsigned char v)
@@ -730,6 +907,7 @@ int _liqcliprect_recursion_depth=0;
 
 void liqcliprect_drawsketch(liqcliprect *self,liqsketch *page,int l,int t,int w,int h,int drawmode)	// 0=preview, 1=detailed, 2=latest point only 4=no aspect
 {
+	
 	if(!page)return;
 	if(l+w<self->sx || t+h<self->sy) return;
 	if(l>=self->ex) return;
@@ -767,6 +945,32 @@ void liqcliprect_drawsketch(liqcliprect *self,liqsketch *page,int l,int t,int w,
 	}
 
 
+	// required for rotation
+	float angle;
+	if(drawmode)
+	{
+		angle=0;
+	}
+	else
+	{
+		if(liqapp_pref_checkexists("autorotate"))
+			angle = page->angle + liqaccel_getangle();
+		else
+			angle = page->angle;
+	}
+	
+	
+	//liqapp_log("ang:   %-3.3f   loc:   %-3.3f",angle,angleloc);
+			   
+	
+	costable_init();
+	
+	
+	// FromCentre used for rotation to know the centre of the from used block
+	int fcx = fox + fmx/2;
+	int fcy = foy + fmy/2;
+
+
 
 
 	int fmap2=fmx*fmx+fmy*fmy;
@@ -863,7 +1067,7 @@ void liqcliprect_drawsketch(liqcliprect *self,liqsketch *page,int l,int t,int w,
 	
 	
 	// Sun Aug 23 12:41:07 2009 lcuk : fuckit, try full res always and see
-	if(liqapp_hardware_product_ispowerful_get())
+	if(liqapp_hardware_product_ispowerful_get() && (angle==0))
 		rpt=1;
 
 	rpt=0;
@@ -955,6 +1159,7 @@ void liqcliprect_drawsketch(liqcliprect *self,liqsketch *page,int l,int t,int w,
 	liqstroke *stroke=page->strokefirst;
 	while(stroke)
 	{
+		liqstroke_hold(stroke);
 		if(stroke->pointcount>=2)
 		{
 			unsigned char y=stroke->pen_y;
@@ -1015,6 +1220,11 @@ void liqcliprect_drawsketch(liqcliprect *self,liqsketch *page,int l,int t,int w,
 
 				p1x=p1->x;	// rotate now..
 				p1y=p1->y;
+				
+				// finally rotating
+				matrot(fcx,fcy, &p1x,&p1y,angle);
+
+
 
 				p2 = p1->linknext;
 				while(p2)
@@ -1023,11 +1233,21 @@ void liqcliprect_drawsketch(liqcliprect *self,liqsketch *page,int l,int t,int w,
 					p2x=p2->x;   // rotate now..
 					p2y=p2->y;
 
+					// finally rotating
+					matrot(fcx,fcy, &p2x,&p2y,angle);
+
+
 					// the heavy math part is here...
 					lsx=tox+((p1x-fox)*tmx/fmx);
 					lsy=toy+((p1y-foy)*tmy/fmy);
 					lex=tox+((p2x-fox)*tmx/fmx);
 					ley=toy+((p2y-foy)*tmy/fmy);
+					
+					// idea:
+					// try rotating the target coords instead of the dest
+					// the thinking here is that it gets me in thinking for the cliprect at the head of the function
+					//matrot(tcx,tcy, &lxs,&lsy,angle);
+					//matrot(tcx,tcy, &les,&ley,angle);
 
 
 					// high definition color scaling :)
@@ -1042,9 +1262,24 @@ void liqcliprect_drawsketch(liqcliprect *self,liqsketch *page,int l,int t,int w,
 					unsigned char sy=	    (char)(      f * (fy    )) ;
 					unsigned char su=	    (char)(128 + f * (fu-128)) ;
 					unsigned char sv=	    (char)(128 + f * (fv-128)) ;
-					liqcliprect_drawlinecolor(self,lsx,lsy,lex,ley,    sy,su,sv);
-
+					
+					
+					
+					liqcliprect_drawlinecolor(self,lsx,lsy,lex,ley,  sy,su,sv);
 					if(isselected) liqcliprect_drawlinecolor(self,lsx+1,lsy+1,lex+1,ley+1,    sy,su,sv);
+					
+					
+					//liqcliprect_drawthicklinecolor(self,lsx,lsy,lex,ley, 5, sy,su,sv);
+					//if(isselected) liqcliprect_drawthicklinecolor(self,lsx+1,lsy+1,lex+1,ley+1, 5,   sy,su,sv);
+					
+					
+					
+					
+					//stroke->pen_thick = 4;		// fakey!
+					//int thick_ratio = (float)stroke->pen_thick * (fmap2/tmap2);
+					//liqapp_log("thick %i,  %3.3f %3.3f %3.3f",thick_ratio, fmap2,tmap2,(fmap2/tmap2));
+					//if(isselected) thick_ratio+=2;
+					//liqcliprect_drawthicklinecolor(self,lsx,lsy,lex,ley,  thick_ratio,  sy,su,sv);
 
 
 
@@ -1178,6 +1413,8 @@ void liqcliprect_drawsketch(liqcliprect *self,liqsketch *page,int l,int t,int w,
 			// supposing we actually said this stroke was a BOX/Circle/Polygon/Triangle/Tile etc
 			// what would we draw then?
 		}
+		
+		liqstroke_release(stroke);
 		stroke=stroke->linknext;
 	}
 }
@@ -1185,6 +1422,12 @@ void liqcliprect_drawsketch(liqcliprect *self,liqsketch *page,int l,int t,int w,
 
 
 
+int		liqcliprect_drawcell(      liqcliprect *self, liqcell *cell,   int x, int y, int w,int h       )
+{
+
+	liqcell_easypaint(cell,self,x,y,w,h);
+	return 0;
+}
 
 
 
@@ -1252,7 +1495,7 @@ static float calcaspect(int captionw,int captionh,int availw,int availh)
 
 
 
-inline void liqcliprect_drawimagecolor(		liqcliprect *self,liqimage *image,int x,int y,int w,int h,int aspectlock)
+void liqcliprect_drawimagecolor(		liqcliprect *self,liqimage *image,int x,int y,int w,int h,int aspectlock)
 {
 
 	if(!w || !h) return;
@@ -1356,7 +1599,7 @@ int b=(y+h);
 
 
 
-inline void liqcliprect_drawimageblendcolor(		liqcliprect *self,liqimage *image,int x,int y,int w,int h,char blend,int aspectlock)
+void liqcliprect_drawimageblendcolor(		liqcliprect *self,liqimage *image,int x,int y,int w,int h,char blend,int aspectlock)
 {
 
 	// 20090408_002917 lcuk : same as drawimagecolor above, but for blended
@@ -1449,4 +1692,65 @@ int b=(y+h);
 								blend
 								);
 
+
+/* this is just thoughts, and was going to implement this here to test making a 9block rounded edge blitter work
+    // the aim is to allow a specified image to be used in a stretch corners concept
+	// to keep rounded edges looking sharp and correct
+	// but allowing a textured core area to stretch easily
+	// test of the special renderer
+
+	int iw=w ;
+	int ih=h ;
+
+	int gw=8;
+	int gh=8;
+	
+	
+	void blit(int cx,int cy,int cw,int ch)
+	{
+		cx+=x;
+		cy+=y;
+		//
+		if(cx+cw < dx){ return; }
+		if(cx    < dx){ cw-=(dx-cx); cx=dx; }
+		if(cx    > dx+dw){ return; }
+		if(cx+cw > dx+dw){ cw=cx-(dx+dw); }
+		if(cy+ch < dy){ return; }
+		if(cy    < dy){ ch-=(dy-cy); cy=dy; }
+		if(cy    > dy+dh){ return; }
+		if(cy+ch > dy+dh){ ch=cy-(dy+dh); }
+		
+		xsurface_drawzoomblendimage( image, cx,cy,cw,ch ,
+							self->surface,
+								//x,y,
+								//w,h
+								cr.sx,cr.sy,
+								cw,ch,
+
+								blend
+								);		
+	}
+
+
+
+	blit(0,    0,      gw,      gh);
+	blit(gw,   0,      iw-gw-gw,gh);
+	blit(iw-gw,0,      gw,      gh);
+
+
+
+	blit(0,    gh,      gw,      ih-gh-gh);
+	blit(gw,   gh,      iw-gw-gw,gh-gh-gh);
+	blit(iw-gw,gh,      gw,      gh-gh-gh);
+
+	blit(0,    ih-gh,   gw,      gh);
+	blit(gw,   ih-gh,   iw-gw-gw,gh);
+	blit(iw-gw,ih-gh,   gw,      gh);
+*/
+
 }
+
+#ifdef __cplusplus
+}
+#endif
+
