@@ -1,9 +1,16 @@
+#include <stdlib.h>
+#include <string.h>
 #include <time.h>			// req for sleep
 #include <sys/time.h>		// req for getticks
 
+#include "liqapp_prefs.h"
 #include "liqcell.h"
 #include "liqcell_prop.h"
 #include "liqcell_easyrun.h"
+#include "liqcell_easypaint.h"
+
+
+// 20100628_0325 lcuk : removing the cover so that liqpostcard works - liqsketch will end up in its own project soon
 
 
 // /usr/share/icons/hicolor/40x40/hildon/qgn_stat_battery_full100.png
@@ -22,7 +29,9 @@
 #include <curl/curl.h>
 
 
-
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 
 
@@ -65,7 +74,7 @@ static size_t curl_memorybuffer(void *ptr, size_t size, size_t nmemb, void *data
 
 
 
-int post_to_liqbase_net(char *filename,char *datakey,int replyid)
+int post_to_liqbase_net(const char *filename, const char *datakey,int replyid)
 {
 	if(!datakey)datakey="liqbase";
  
@@ -74,9 +83,9 @@ int post_to_liqbase_net(char *filename,char *datakey,int replyid)
 
 	char *username = app.username;
 	char *userpassmd5 = liqapp_pref_getvalue("userpassmd5");
-	if(!userpassmd5 || !*userpassmd5)
+	if(!userpassmd5 || !*userpassmd5 || !username || !*username)
 	{
-		liqapp_log("post_to_liqbase_net not performed, no userpass configured");
+		liqapp_log("post_to_liqbase_net not performed, no username or pass configured");
 		return -1;
 	}
     
@@ -173,7 +182,6 @@ int post_to_liqbase_net(char *filename,char *datakey,int replyid)
 			{
 				char filedate[256];	
 				liqapp_formatnow(filedate,sizeof(filedate),"yyyymmdd_hhmmss");
-				char *filetitle =liqapp_filename_walkoverpath(fn);
 				// got a file
 				char s[FILENAME_MAX*3];
 				snprintf(s,sizeof(s),"mv %s %s.old.%s",fn,fn,filedate);
@@ -191,7 +199,7 @@ int post_to_liqbase_net(char *filename,char *datakey,int replyid)
 		}
 		
 		liqcell *notes = liqcell_child_lookup(self,"notes");
-		char *key;
+		const char *key;
 		
 		
 		if(notes)
@@ -242,7 +250,7 @@ int post_to_liqbase_net(char *filename,char *datakey,int replyid)
         {
             // no parent..
             // make sure we show the cover
-            liqcell_setvisible(cover,1);
+            liqcell_setvisible(cover,0);
         }
         else
         {
@@ -250,6 +258,8 @@ int post_to_liqbase_net(char *filename,char *datakey,int replyid)
             // make sure we hide the cover
             liqcell_setvisible(cover,0);
         }
+        
+        return 0;
     }
 
 	static int liqsketchedit_dialog_open(liqcell *self, liqcelleventargs *args, void *context)
@@ -268,6 +278,8 @@ int post_to_liqbase_net(char *filename,char *datakey,int replyid)
             // make sure we hide the cover
             liqcell_setvisible(cover,0);
         }
+        
+        return 0;
     }
 
 
@@ -281,7 +293,7 @@ int post_to_liqbase_net(char *filename,char *datakey,int replyid)
         {
             // no parent..
             // make sure we show the cover
-            liqcell_setvisible(cover,1);
+            liqcell_setvisible(cover,0);
         }
         else
         {
@@ -325,7 +337,7 @@ int post_to_liqbase_net(char *filename,char *datakey,int replyid)
 		
 		liqsketch_strokeremove(sketch,sketch->strokelast);
 
-		liqcell_settag(editor,1);
+		liqcell_settag(editor, (void*)1);
 		
 		liqcell_handlerrun(editor,"undo",NULL);
 
@@ -387,7 +399,6 @@ int post_to_liqbase_net(char *filename,char *datakey,int replyid)
 			{
 				char filedate[256];	
 				liqapp_formatnow(filedate,sizeof(filedate),"yyyymmdd_hhmmss");
-				char *filetitle =liqapp_filename_walkoverpath(fn);
 				// got a file
 				char s[FILENAME_MAX*3];
 				snprintf(s,sizeof(s),"mv %s %s.del.%s",fn,fn,filedate);
@@ -398,7 +409,7 @@ int post_to_liqbase_net(char *filename,char *datakey,int replyid)
 		
 		liqcell_setsketch(editor,NULL);
 		
-		liqcell_settag(editor,1);
+		liqcell_settag(editor, (void*)1);
 
 
 		liqcell_handlerrun(editor,"cleared",NULL);
@@ -448,8 +459,13 @@ int post_to_liqbase_net(char *filename,char *datakey,int replyid)
 					//s->pixelwidth =liqcell_getw(self);
 					//s->pixelheight=liqcell_geth(self);
 					
-					s->pixelwidth =liqcanvas_getwidth(self);
-					s->pixelheight=liqcanvas_getheight(self);
+					
+					//s->pixelwidth =liqcanvas_getwidth();
+					//s->pixelheight=liqcanvas_getheight();
+					liqimage *img = canvas.surface;				// this might be different to the answers liqcanvas_getwidth() returns
+					s->pixelwidth =liqimage_getwidth(img);		// this isnt best solution, todo review again
+					s->pixelheight=liqimage_getheight(img);		// 
+					
 					
 					s->dpix=225;	// damn, dont like using this here
 					s->dpiy=225;
@@ -460,7 +476,7 @@ int post_to_liqbase_net(char *filename,char *datakey,int replyid)
 		// this was only to see high intensity quickly
 		//args->mez/=2;
 		
-		liqapp_log("sss me %i,%i   o %i,%i   ss %i,%i",args->mex,args->mey,    args->ox,args->oy,   args->mex-args->ox,args->mey-args->oy);
+		//liqapp_log("sss me %i,%i   o %i,%i   ss %i,%i",args->mex,args->mey,    args->ox,args->oy,   args->mex-args->ox,args->mey-args->oy);
 
 		if(args->mcnt==1)
 		{
@@ -518,7 +534,7 @@ int post_to_liqbase_net(char *filename,char *datakey,int replyid)
 			}
 		}
 		// 20090421_232509 lcuk : make sure we mark ourselves as dirty
-		liqcell_settag(self,1);
+		liqcell_settag(self, (void*)1);
 
 		return 1;
 	}
@@ -539,16 +555,23 @@ static int liqsketchedit_resize(liqcell *self, liqcelleventargs *args, void *con
 	
 	int ww=liqcell_getw(self);
 	int hh=liqcell_geth(self);
-	
+/*
 	liqcell_setrect(cover,  0,  0,            ww*1.0,hh*1.0);
 	liqcell_setrect(undo,  ww*0.9,  0,        ww*0.1,hh*0.3);
 	liqcell_setrect(clear, ww*0.9,  hh*0.3,   ww*0.1,hh*0.3);
 	liqcell_setrect(save , ww*0.9,  hh*0.6,   ww*0.1,hh*0.3);
 	liqcell_setrect(del ,  ww*0.9,  hh*0.9,   ww*0.1,hh*0.1);
 	liqcell_setrect(notes ,  ww*0.25,  hh*1,   ww*0.5,hh*0.1);
-	
-	
-	
+*/
+	// rotation patch test
+	liqcell_setrect(cover,  0,  0,            ww*1.0,hh*1.0);
+	liqcell_setrect(undo,  ww*0.0,  hh*0.9,   ww*0.3,hh*0.1);
+	liqcell_setrect(clear, ww*0.3,  hh*0.9,   ww*0.3,hh*0.1);
+	liqcell_setrect(save , ww*0.6,  hh*0.9,   ww*0.3,hh*0.1);
+	liqcell_setrect(del ,  ww*0.9,  hh*0.9,   ww*0.1,hh*0.1);
+	liqcell_setrect(notes ,  ww*0.25,  hh*1,   ww*0.5,hh*0.1);
+		
+	return 0;
 }
 
 	static int liqsketchedit_paint(liqcell *self, liqcellpainteventargs *args,liqcell *liqsketchedit)
@@ -576,6 +599,8 @@ static int liqsketchedit_resize(liqcell *self, liqcelleventargs *args, void *con
 				liqcell_setdirty(liqsketchedit,1);
 			}
 		}
+		
+		return 0;
 	}
 
 
@@ -609,33 +634,33 @@ liqcell *liqsketchedit_create()
 
 		b = liqcell_quickcreatevis("undo","button",  800-180,20,   160,160 );
 		liqcell_setfont(   b, liqfont_cache_getttf("/usr/share/fonts/nokia/nosnb.ttf", (24), 0) );
-		liqcell_handleradd(b,    "click",   liqsketchedit_undo_click);
+		liqcell_handleradd(b,    "click",   (void*)liqsketchedit_undo_click);
 		liqcell_propsets(  b,    "backcolor", "xrgb(100,0,100)" );
-		liqcell_handleradd(b,    "mouse",   liqsketchedit__cmdnull_mouse);
+		liqcell_handleradd(b,    "mouse",   (void*)liqsketchedit__cmdnull_mouse);
 		liqcell_child_insert( self, b );
 
 
 		b = liqcell_quickcreatevis("clear","button",  800-180,20,   160,160 );
 		liqcell_setfont(   b, liqfont_cache_getttf("/usr/share/fonts/nokia/nosnb.ttf", (24), 0) );
-		liqcell_handleradd(b,    "click",   liqsketchedit_clear_click);
+		liqcell_handleradd(b,    "click",   (void*)liqsketchedit_clear_click);
 		liqcell_propsets(  b,    "backcolor", "xrgb(0,0,100)" );
-		liqcell_handleradd(b,    "mouse",   liqsketchedit__cmdnull_mouse);
+		liqcell_handleradd(b,    "mouse",   (void*)liqsketchedit__cmdnull_mouse);
 		liqcell_child_insert( self, b );
 	
 
 		b = liqcell_quickcreatevis("save","button",  800-180,200,   160,160 );
 		liqcell_setfont(   b, liqfont_cache_getttf("/usr/share/fonts/nokia/nosnb.ttf", (24), 0) );
-		liqcell_handleradd(b,    "click",   liqsketchedit_save_click);
+		liqcell_handleradd(b,    "click",   (void*)liqsketchedit_save_click);
 		liqcell_propsets(  b,    "backcolor", "xrgb(0,100,0)" );
-		liqcell_handleradd(b,    "mouse",   liqsketchedit__cmdnull_mouse);
+		liqcell_handleradd(b,    "mouse",   (void*)liqsketchedit__cmdnull_mouse);
 		liqcell_child_insert( self, b );
 
 
 		b = liqcell_quickcreatevis("del","button",  800-180,200,   160,160 );
 		liqcell_setfont(   b, liqfont_cache_getttf("/usr/share/fonts/nokia/nosnb.ttf", (24), 0) );
-		liqcell_handleradd(b,    "click",   liqsketchedit_del_click);
+		liqcell_handleradd(b,    "click",   (void*)liqsketchedit_del_click);
 		liqcell_propsets(  b,    "backcolor", "xrgb(100,0,0)" );
-		liqcell_handleradd(b,    "mouse",   liqsketchedit__cmdnull_mouse);
+		liqcell_handleradd(b,    "mouse",   (void*)liqsketchedit__cmdnull_mouse);
 		liqcell_child_insert( self, b );
 
 
@@ -654,16 +679,20 @@ liqcell *liqsketchedit_create()
 
 		liqsketchedit_resize(self,NULL, NULL);
 
-		//liqcell_handleradd_withcontext(self,    "keypress",   liqsketchedit_keypress,   self);
-        liqcell_handleradd_withcontext(self,    "shown",   liqsketchedit_shown,   self);
-		liqcell_handleradd_withcontext(self,    "dialog_open",   liqsketchedit_dialog_open,   self);
-		liqcell_handleradd_withcontext(self,    "dialog_close",   liqsketchedit_dialog_close,   self);
-		liqcell_handleradd_withcontext(self,    "mouse",   liqsketchedit_mouse,   self);
-		liqcell_handleradd_withcontext(self,    "resize",   liqsketchedit_resize,   self);
-		liqcell_handleradd_withcontext(self,    "paint",   liqsketchedit_paint,   self);
+        liqcell_handleradd_withcontext(self,    "shown",   (void*)liqsketchedit_shown,   self);
+		liqcell_handleradd_withcontext(self,    "dialog_open",   (void*)liqsketchedit_dialog_open,   self);
+		liqcell_handleradd_withcontext(self,    "dialog_close",   (void*)liqsketchedit_dialog_close,   self);
+		liqcell_handleradd_withcontext(self,    "mouse",   (void*)liqsketchedit_mouse,   self);
+		liqcell_handleradd_withcontext(self,    "resize",   (void*)liqsketchedit_resize,   self);
+		liqcell_handleradd_withcontext(self,    "paint",   (void*)liqsketchedit_paint,   self);
 		
 		
 	}
 	return self;
 
 }
+
+#ifdef __cplusplus
+}
+#endif
+
