@@ -3,11 +3,15 @@
 
 
 #include "liqbase.h"
+#include "liqapp_prefs.h"
 
 #include "liqx11info.h"
 
 #include "liqx11overlay.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 //############################################################# keyboard bits
 
@@ -29,9 +33,78 @@ int liqx11_modifierprev=0;
 int liqx11_modifier=0;
 
 
+XImage *cover_image = NULL;
+XImage *liqimage_convert_to_ximage(liqimage *self, Display *dis, int screen);
+
+void cover_image_release(liqx11info *myx11info)
+{
+
+	if(cover_image)
+	{
+		XDestroyImage(cover_image);
+		//XFree(cover_image);
+		cover_image=NULL;
+	}
+	
+}
+	
+void cover_image_rebuild(liqx11info *myx11info)
+{
+	// if we have been passed in as livewp mode
+	// then bail here, we do not need backing image
+	if(liqapp_getopt_exist("livewp"))
+		return;
+		
+	cover_image_release(myx11info);
+	
+	cover_image = liqimage_convert_to_ximage( canvas.rotation_native_surface,  myx11info->mydisplay , myx11info->myscreen);
+	
+}
+	
+void cover_image_blit(liqx11info *myx11info)
+{
+
+	if(cover_image)
+	{
+		liqapp_log("blit cover");
+		liqimage *img = canvas.rotation_native_surface;
+		XPutImage (myx11info->mydisplay, myx11info->mywindow, myx11info->mygc, cover_image, 0, 0, 0, 0, liqimage_getwidth(img), liqimage_getheight(img) );
+		XFlush (myx11info->mydisplay);
+	}
+	
+}
 
 //############################################################# 
 
+int x11_get_first_xvport(Display *display)
+{
+	
+	
+	
+	//################################################# get adapter info and find our port
+	int             ret;
+	unsigned int	p_num_adaptors;
+	XvAdaptorInfo*  ai;
+	
+
+	ret = XvQueryAdaptors(display, DefaultRootWindow(display), &p_num_adaptors, &ai);  
+	if (ret != Success) 
+	{
+		{ return liqapp_errorandfail(-1,"x11_get_first_xvport XvQueryAdaptors failed"); }
+	}
+
+	if(p_num_adaptors==0) 
+	{
+		{ return liqapp_errorandfail(-1,"x11_get_first_xvport XvQueryAdaptors returned no adapters"); }
+	}
+    
+    liqapp_log("x11_get_first_xvport Xv p_num_adaptors=%i  ai[0].base_id=%d",p_num_adaptors, ai[0].base_id);
+	
+	return ai[0].base_id;
+	
+}
+
+
 static void x11_set_fullscreen_state(Display *display, Window window, int action)
 {
     XEvent xev;
@@ -48,7 +121,8 @@ static void x11_set_fullscreen_state(Display *display, Window window, int action
     xev.xclient.data.l[3]	= 0;
     xev.xclient.data.l[4]	= 0;
     /* finally send that damn thing */
-    if( !XSendEvent(display, DefaultRootWindow(display), False, SubstructureRedirectMask | SubstructureNotifyMask, &xev) )
+  //  if( !XSendEvent(display, DefaultRootWindow(display), False, SubstructureRedirectMask | SubstructureNotifyMask, &xev) )
+    if( !XSendEvent(display, DefaultRootWindow(display), False, SubstructureRedirectMask , &xev) )
 	{
 		{ liqapp_errorandfail(-1,"canvas x11 could not set fullscreen"); }
     }
@@ -66,6 +140,8 @@ int liqx11info_regrab_focus(liqx11info *myx11info)
 	{
 		{ liqapp_errorandfail(-1,"x11info XRaiseWindow, could not raise"); }
 	}
+	
+	return 0;
 }
 
 
@@ -99,6 +175,7 @@ int liqx11info_init(liqx11info *myx11info, int pixelwidth,int pixelheight,int fu
 	myx11info->myoverlay     = NULL;
 	myx11info->mydisplay     = XOpenDisplay("");
 	myx11info->myscreen      = DefaultScreen(myx11info->mydisplay);
+	myx11info->mywindow      = 0;//NULL;
 	
 	
 	myx11info->myinnotifyflag=0;
@@ -128,8 +205,36 @@ int liqx11info_init(liqx11info *myx11info, int pixelwidth,int pixelheight,int fu
 	}
 	
 	
+
+
+
+XvPortID  xvport_num = x11_get_first_xvport(myx11info->mydisplay);
+
+#ifdef USE_MAEMO
+int       xvoverlaycolorkey=-1;
+	// This code barfs on the desktop for me (Robin), nvidia drivers.. so make it Maemo-only.
+        Atom xv_colorkey = XInternAtom(myx11info->mydisplay, "XV_COLORKEY", 0);
+        XvGetPortAttribute(myx11info->mydisplay, xvport_num, xv_colorkey, &xvoverlaycolorkey);
+		
+		
+		
+int       xvautopaintcolorkey=-1;
+
+        Atom xv_autopaintcolorkey = XInternAtom(myx11info->mydisplay, "XV_AUTOPAINT_COLORKEY", 0);
+        XvGetPortAttribute(myx11info->mydisplay, xvport_num, xv_autopaintcolorkey, &xvautopaintcolorkey);
+
+
+	liqapp_log("xv_colorkey %d,%d",xvoverlaycolorkey,xvautopaintcolorkey);
+	xvoverlaycolorkey=1;
+	XvSetPortAttribute(myx11info->mydisplay, xvport_num, XInternAtom(myx11info->mydisplay, "XV_COLORKEY", True), xvoverlaycolorkey );
+
+
+	xvautopaintcolorkey=1;
+	XvSetPortAttribute(myx11info->mydisplay, xvport_num, XInternAtom(myx11info->mydisplay, "XV_AUTOPAINT_COLORKEY", True), xvautopaintcolorkey );
 	
-	
+#endif
+
+	//################################################# 
 	
 	
 	liqapp_log("x11info preparing baselines and hint");
@@ -140,6 +245,13 @@ int liqx11info_init(liqx11info *myx11info, int pixelwidth,int pixelheight,int fu
 	
 		mybackground  = BlackPixel(myx11info->mydisplay, myx11info->myscreen);
 		myforeground  = WhitePixel(myx11info->mydisplay, myx11info->myscreen);
+		
+#ifdef USE_MAEMO
+		// See above. We don't use xvoverlaycolorkey off Maemo.
+		liqapp_log("colors Back=%d fore=%d key=%d",mybackground,myforeground,xvoverlaycolorkey);
+		
+		mybackground = xvoverlaycolorkey;
+#endif
 
 
 	
@@ -198,8 +310,9 @@ int liqx11info_init(liqx11info *myx11info, int pixelwidth,int pixelheight,int fu
 	// ################### good x11 info
 	// http://www.openmash.org/lxr/source/xlib/X11/X.h?c=tk8.3#L99
 	// http://static.cray-cyber.org/Documentation/NEC_SX_R10_1/G1AE02E/CHAP10.HTML
-	
-	
+
+
+
  	//################################################# init normal x11 handler mask
 	// set the actual event mask we normally want for this window
 	XSelectInput(	myx11info->mydisplay,myx11info->mywindow,
@@ -233,40 +346,39 @@ int liqx11info_init(liqx11info *myx11info, int pixelwidth,int pixelheight,int fu
     myx11info->my_WM_DELETE_WINDOW = XInternAtom(myx11info->mydisplay, "WM_DELETE_WINDOW", True);
     XSetWMProtocols(myx11info->mydisplay, myx11info->mywindow, &myx11info->my_WM_DELETE_WINDOW, 1);
 
-
-
+		
 	
 	//################################################# lets make sure we are top of the pile
 	
 	XMapRaised(		myx11info->mydisplay,myx11info->mywindow);
-	
-	
-		XEvent event;
-		do 
-		{
-			XNextEvent(myx11info->mydisplay, &event);
-		}
-		while (event.type != MapNotify || event.xmap.event != myx11info->mywindow);
-	
+		
 	
 	if(fullscreen)
+	{
+		liqapp_log("x11info going fullscreen");
 		x11_set_fullscreen_state(myx11info->mydisplay,myx11info->mywindow,1);
-	//isfullscreen=1;	
-	
-	
+	}
+
+	XEvent event;
+	do 
+	{
+		XNextEvent(myx11info->mydisplay, &event);
+		liqapp_log("looping %i  %i",event.type,MapNotify);
+	}
+	while (event.type != MapNotify || event.xmap.event != myx11info->mywindow);
+
 	//################################################# put the overlay in place :)
-	
 
 	liqapp_log("x11info allocating overlay");
 	
 	myx11info->myoverlay = &myx11info->myoverlaycore;
-	liqx11overlay_init(myx11info->myoverlay, myx11info->mydisplay,myx11info->myscreen,myx11info->mywindow,myx11info->mygc);
+	liqx11overlay_init(myx11info->myoverlay, myx11info->mydisplay,myx11info->myscreen,myx11info->mywindow,myx11info->mygc,  canvas.pixelwidth,canvas.pixelheight);
+	
+	liqapp_log("x11info binding overlay to window");
+	myx11info->myoverlay->window = myx11info->mywindow;
+
+	liqapp_log("x11info showing overlay");
 	liqx11overlay_show(myx11info->myoverlay);
-	liqx11overlay_refreshdisplay(myx11info->myoverlay);
-
-
-    liqx11info_regrab_focus(myx11info);
-
 
 	
 	return 0;
@@ -284,9 +396,7 @@ int liqx11info_init(liqx11info *myx11info, int pixelwidth,int pixelheight,int fu
 int liqx11info_close(liqx11info *myx11info)
 {
 	
-	
-	
-	liqapp_log("shutting down");
+	liqapp_log("x11 shutting down");
 	
 	
 
@@ -312,8 +422,6 @@ int liqx11info_close(liqx11info *myx11info)
 
 int liqx11info_eventgetcount(liqx11info *myx11info)
 {
-//	return XPending(myx11info->mydisplay);
-
 	int evc=XEventsQueued(myx11info->mydisplay, QueuedAfterFlush);
 	//liqapp_log("evc %i",evc);
 	return evc;
@@ -343,7 +451,7 @@ int liqx11info_eventgetnext(liqx11info *myx11info,XEvent *event)
 
 int liqx11info_refreshdisplay(liqx11info *myx11info)
 {
-    
+ //   liqapp_log("liqx11info_refreshdisplay %d", myx11info->myoverlay->yuv_shminfo_attached);
 /*    
     XImage *liqimage_convert_to_ximage(liqimage *self, Display *dis, int screen);
  
@@ -359,11 +467,63 @@ int liqx11info_refreshdisplay(liqx11info *myx11info)
 		XFlush(myx11info->mydisplay);
         XDestroyImage(img);
     }
- */    
+ */
+
+
+	if((canvas.surface) && (canvas.surface != canvas.rotation_native_surface) )
+	{
+		//liqapp_log("liqx11info_refreshdisplay rotating!");
+		
+		liqimage_rotate( canvas.rotation_native_surface, canvas.surface, 90 );
+	}
+
+	if(cover_image)
+	{
+		liqapp_log("liqx11info_refreshdisplay2");
+		
+		cover_image_rebuild(myx11info);
+		cover_image_blit(myx11info);
+		
+		return 0;
+	}
+
+
 	return liqx11overlay_refreshdisplay(myx11info->myoverlay);
 }
 
 
+// x11
+
+static int x11_event_mouse_coord_convert(liqx11info *myx11info,XEvent *xev)
+{
+	// internal function used by the core x11 handler
+	// to rotate and reorient/scale the data coming from system
+	// into what we need internally.
+int xres   = ScreenOfDisplay (myx11info->mydisplay, DefaultScreen(myx11info->mydisplay))->width;
+int yres   = ScreenOfDisplay (myx11info->mydisplay, DefaultScreen(myx11info->mydisplay))->height;
+
+
+	//if(x11_seeevent)liqapp_log("x11.event.motionnot.OR xy %d,%d   can wh %d,%d",xev.xmotion.x,xev.xmotion.y,  canvas.pixelwidth,canvas.pixelheight);
+	if(canvas.rotation_angle==90)
+	{
+		if(canvas.fullscreen)
+		{
+			xev->xmotion.x= xev->xmotion.x * canvas.rotation_native_surface->width  / xres;
+			xev->xmotion.y= xev->xmotion.y * canvas.rotation_native_surface->height / yres;							}
+			//if(x11_seeevent)liqapp_log("x11.event.motionnot.md xy %d,%d   can wh %d,%d",xev.xmotion.x,xev.xmotion.y,  canvas.pixelwidth,canvas.pixelheight);
+			
+		{ int t=xev->xmotion.x; xev->xmotion.x=canvas.rotation_native_surface->height-1-xev->xmotion.y; xev->xmotion.y=t; }
+	}
+	else
+	{
+		if(canvas.fullscreen)
+		{
+						xev->xmotion.x= xev->xmotion.x * canvas.pixelwidth  / xres;
+						xev->xmotion.y= xev->xmotion.y * canvas.pixelheight / yres;
+		}
+	}
+	return 0;
+}
 
 
 
@@ -377,14 +537,48 @@ int liqx11info_refreshdisplay(liqx11info *myx11info)
 
 
 
+static int liqx11info_overlay_bring_online(liqx11info *myx11info)
+{
+                        if(!myx11info->myoverlay)
+                        {
+                        //	liqapp_sleep(100);
+                            myx11info->myoverlay = &myx11info->myoverlaycore;
+                            liqx11overlay_init(myx11info->myoverlay, myx11info->mydisplay,myx11info->myscreen,myx11info->mywindow,myx11info->mygc,    canvas.pixelwidth,canvas.pixelheight);
+                            //liqx11overlay_show(myx11info->myoverlay);
+                            //isbusyrendering=0;
+                        }
+						
+						if((myx11info->myoverlay))
+						{
+							liqx11overlay_show(myx11info->myoverlay);
+							
+							//isbusyrendering=1;
+							
+							cover_image_release(myx11info);
+						}
+
+						myx11info->myisvisibleflag=1;
+						myx11info->myisfocusflag=1;
+						
+						return 0;
+}
 
 
 
 
-
-
-
-
+int liqx11info_minimize(liqx11info *myx11info)
+{
+	
+						if((myx11info->myoverlay))
+						{
+							cover_image_rebuild(myx11info);
+							cover_image_blit(myx11info);
+							liqx11overlay_hide(myx11info->myoverlay);
+							//liqx11overlay_refreshdisplay(myx11info->myoverlay);
+						}
+						
+						return 0;	
+}
 
 
 int liqx11info_get_next_liqevent(liqx11info *myx11info,LIQEVENT *ev,int *dirtyflagptr)
@@ -398,14 +592,21 @@ unsigned long ticksnow=ev->ticks;
 //  goto foo;
 //	canvas.keepalivealarmtime=0;
 //  ok we are here.  we are not sure if there are any events
+int sleeptime=25;
 	while( ((canvas.keepalivealarmtime>0) && (liqx11info_eventgetcount(myx11info)==0)) )
 	{
 		// we have been called with NO events pending.
 		// we must sleep
 		//liqapp_log("Sleeping...%u,%u dif=%u  kat=%u",ev->ticks,ticksnow,ticksnow-ev->ticks,canvas.keepalivealarmtime);
-		liqapp_sleep(25);
+		
+		liqapp_sleep(sleeptime);
+		//liqapp_sleep(100);
+		
 		if(liqcanvas_eventcount()>0) break;
 		
+		
+		if(sleeptime<500)sleeptime+=25;
+		
 		if((dirtyflagptr) && (*dirtyflagptr))
 		{
 			//liqapp_log("dirty flag bailout!!");
@@ -445,8 +646,14 @@ unsigned long ticksnow=ev->ticks;
 	}
 	
 	
+	// pan out, visibility gone
+	// THEN(1)pan to other app, focus gone too
+	// THEN(2)pan back in, configurenotify should restore visibility
 	
 	
+	
+	//myx11info->myisvisibleflag = 1;
+	
 foo:
 
 	//liqx11info_eventgetnext();
@@ -456,13 +663,15 @@ foo:
 	
 	liqx11info_eventgetnext( myx11info, &xev );
 	
-	
-	
+	//http://linux.die.net/man/3/xanyevent
+const int x11_seeevent=0;
+
 	int keylen=0;
-	//liqapp_log("xNextEvent %i",xev.type);
-	
-	
+	if(x11_seeevent)liqapp_log("x11.NextEvent %i  dpy=%d window=%d root=%d subwindow=%d,   vis=%d foc=%d",
+			   xev.type,xev.xany.display,xev.xany.window, xev.xcrossing.root,xev.xcrossing.subwindow,
+			   myx11info->myisvisibleflag,myx11info->myisfocusflag);
 	
+
 	
 	
 			switch (xev.type)
@@ -472,25 +681,25 @@ foo:
 				case ClientMessage:
 					if (xev.xclient.data.l[0] == myx11info->my_WM_DELETE_WINDOW)
 					{
-						liqapp_log("event.WM_DELETE_WINDOW");
+						if(x11_seeevent)liqapp_log("x11.event.WM_DELETE_WINDOW");
 						// deleting window
 						// done=1;
 						// Sat Aug 29 20:54:44 2009 lcuk : find much cleaner way to handle this
 						exit(0);
 						break;
 					}
-					liqapp_log("event.ClientMessage");
+					if(x11_seeevent)liqapp_log("x11.event.ClientMessage");
 					break;
 					
 						
 				//############################################ Child lifetime events 
 	
 				case CreateNotify:
-					liqapp_log("event.CreateNotify");
+					if(x11_seeevent)liqapp_log("x11.event.CreateNotify");
 					break;
 				
 				case DestroyNotify:
-					liqapp_log("event.DestroyNotify");
+					if(x11_seeevent)liqapp_log("x11.event.DestroyNotify");
 					break;			
 				
 				//############################################ Notify events: principle actors in arranging the window 
@@ -499,49 +708,130 @@ foo:
 					// http://tronche.com/gui/x/xlib/events/window-state-change/configure.html
 					// The X server can report ConfigureNotify events to clients wanting information about actual changes to a window's state,
 					// such as size, position, border, and stacking order. 
-					liqapp_log("event.ConfigureNotify");
+					if(x11_seeevent)liqapp_log("x11.event.ConfigureNotify (%i,%i)-step(%i,%i) above=%d overrideredirect=%d",
+							   xev.xconfigure.x,xev.xconfigure.y,xev.xconfigure.width,xev.xconfigure.height,
+							   xev.xconfigure.above,xev.xconfigure.override_redirect);
 					
-					if(myx11info->myoverlay)
-					{				
-						//liqapp_sleep(100);
-
-						//xev.xconfigure.width
-						liqx11overlay_close(myx11info->myoverlay);
-						myx11info->myoverlay = NULL;
-						//isbusyrendering=0;
-
-
-					}
-	
-					if(!myx11info->myoverlay)
+					if( myx11info->myisvisibleflag==0 && myx11info->myisfocusflag==0)
 					{
-					//	liqapp_sleep(100);
-						myx11info->myoverlay = &myx11info->myoverlaycore;
-						liqx11overlay_init(myx11info->myoverlay, myx11info->mydisplay,myx11info->myscreen,myx11info->mywindow,myx11info->mygc);
-						liqx11overlay_show(myx11info->myoverlay);
-						//isbusyrendering=0;
+						break;
+					}					
+					if( myx11info->myisvisibleflag==0 && myx11info->myisfocusflag==1)
+					{
+						if(x11_seeevent)liqapp_log("x11.eep");
+							if(myx11info->myoverlay)
+							{				
+								//liqapp_sleep(100);
+								//xev.xconfigure.width
+								liqx11overlay_close(myx11info->myoverlay);
+								myx11info->myoverlay = NULL;
+								//isbusyrendering=0;
+							}
+							if(!myx11info->myoverlay)
+							{
+							//	liqapp_sleep(100);
+								myx11info->myoverlay = &myx11info->myoverlaycore;
+								liqx11overlay_init(myx11info->myoverlay, myx11info->mydisplay,myx11info->myscreen,myx11info->mywindow,myx11info->mygc,   canvas.pixelwidth,canvas.pixelheight);
+								//isbusyrendering=0;
+							}
+							liqx11overlay_show(myx11info->myoverlay);
+							myx11info->myisvisibleflag = 1;
+							cover_image_release(myx11info);			
+							ev->type = LIQEVENT_TYPE_RESIZE;
+							break;
+					}
+					
+					if( myx11info->myisvisibleflag==0 && myx11info->myisfocusflag==0)
+					{
+						myx11info->myisvisibleflag=1;
+					}
+
+				
+					
+					//if( !myx11info->myisvisibleflag ) break;
+					
+					if(xev.xconfigure.override_redirect) break;
+					
+					if(xev.xconfigure.window == myx11info->mywindow)
+					{
+						if(xev.xconfigure.event == myx11info->mywindow)
+						{
+							// perfect, its just us
+						}
+						else
+							break;
+					}
+					else
+						break;
+					
+					//liqapp_log("green shortcut"); break;
+					
+					//if( xev.xconfigure.above==0 || xev.xconfigure.above==myx11info->mywindow)
+					//{
+					//	myx11info->myisfocusflag=1;
+					//}
+
+
+					
+					if( ( xev.xconfigure.width == liqcanvas_getwidth() ) &&
+					    ( xev.xconfigure.height == liqcanvas_getheight() ) &&
+						( myx11info->myoverlay ) 
+					  )
+					{
+						//ev->type = LIQEVENT_TYPE_RESIZE;
+						//break;
+						// already at correct dimensions
+					}
+					else
+					{
+							
+						if( myx11info->myisfocusflag )
+						{
+					
+							if(myx11info->myoverlay)
+							{				
+								//liqapp_sleep(100);
+								//xev.xconfigure.width
+								liqx11overlay_close(myx11info->myoverlay);
+								myx11info->myoverlay = NULL;
+								//isbusyrendering=0;
+							}
+			
+							if(!myx11info->myoverlay)
+							{
+							//	liqapp_sleep(100);
+								myx11info->myoverlay = &myx11info->myoverlaycore;
+								liqx11overlay_init(myx11info->myoverlay, myx11info->mydisplay,myx11info->myscreen,myx11info->mywindow,myx11info->mygc,   canvas.pixelwidth,canvas.pixelheight);
+								//isbusyrendering=0;
+							}
+						}
 					}
 					//dirty=1;
 					
 					
+				if( myx11info->myisfocusflag )
+				{
+					
+					liqx11overlay_show(myx11info->myoverlay);
+					myx11info->myisvisibleflag = 1;
+					cover_image_release(myx11info);
+				}					
 					ev->type = LIQEVENT_TYPE_RESIZE;
-					
-					
 					// need to update the cr and stuff
 	
 					
 					break;
 				
 				case ReparentNotify:
-					liqapp_log("event.ReparentNotify");
+					if(x11_seeevent)liqapp_log("x11.event.ReparentNotify");
 					break;
 				
 				case MapNotify:
-					liqapp_log("event.MapNotify");
+					if(x11_seeevent)liqapp_log("x11.event.MapNotify");
 					break;
 				
 				case MappingNotify:
-					liqapp_log("event.MappingNotify");
+					if(x11_seeevent)liqapp_log("x11.event.MappingNotify");
 					break;				
 				
 				
@@ -549,43 +839,87 @@ foo:
 	
 	
 				case VisibilityNotify:
-					liqapp_log("event.VisibilityNotify state=%i",xev.xvisibility.state);
-					
+					if(x11_seeevent)liqapp_log("x11.event.VisibilityNotify state=%i",xev.xvisibility.state);
 					myx11info->myisvisibleflag = (xev.xvisibility.state == 0) || (xev.xvisibility.state==1);
-						
-					if((myx11info->myoverlay))
+					if( myx11info->myisvisibleflag )
 					{
-						liqx11overlay_show(myx11info->myoverlay);
-						liqx11overlay_refreshdisplay(myx11info->myoverlay);
-						//isbusyrendering=1;
-					}
-					break;				
-		
-				//############################################ focus
-				case EnterNotify:					
-					liqapp_log("event.EnterNotify");
-
-                        if(!myx11info->myoverlay)
-                        {
-                        //	liqapp_sleep(100);
-                            myx11info->myoverlay = &myx11info->myoverlaycore;
-                            liqx11overlay_init(myx11info->myoverlay, myx11info->mydisplay,myx11info->myscreen,myx11info->mywindow,myx11info->mygc);
-                            //liqx11overlay_show(myx11info->myoverlay);
-                            //isbusyrendering=0;
-                        }
-						
+					
 						if((myx11info->myoverlay))
 						{
 							liqx11overlay_show(myx11info->myoverlay);
 							liqx11overlay_refreshdisplay(myx11info->myoverlay);
 							//isbusyrendering=1;
 						}
+						cover_image_release(myx11info);
+						
+						// visible so make this happen..
+						//myx11info->myisfocusflag=1;
+						// occurs when panout
+					}
+					else
+					{
+						if((myx11info->myoverlay))
+						{
+							cover_image_rebuild(myx11info);
+							cover_image_blit(myx11info);
+							liqx11overlay_hide(myx11info->myoverlay);
+							//liqx11overlay_refreshdisplay(myx11info->myoverlay);
+						}
+						// not visible so force this..
+						//myx11info->myisfocusflag=0;
+						
+					}
+					break;				
+		
+				//############################################ focus
+				case EnterNotify:
+					//http://tronche.com/gui/x/xlib/events/window-entry-exit/
+					
+					if(x11_seeevent)liqapp_log("x11.event.EnterNotify focus=%d state=%d mode=%d detail=%d same_screen=%d window=%d subwindow=%d",
+								xev.xcrossing.focus,
+								xev.xcrossing.state,
+								xev.xcrossing.mode,
+								xev.xcrossing.detail,
+								xev.xcrossing.same_screen,
+								xev.xcrossing.window,
+								xev.xcrossing.subwindow);
+					
+					/*
+					if( (xev.xcrossing.focus==0) && (myx11info->myisfocusflag==0))
+					{
+						// we are potentially being brought back to life
+						// strongly dislike all this on/off
+						//liqx11info_regrab_focus(myx11info);
+						break;
+					}
+					else
+					{
+						if( !myx11info->myisvisibleflag ) break;
+					}
+					*/
+
+					if( myx11info->myisvisibleflag==0 && myx11info->myisfocusflag==0)
+					{
+						myx11info->myisvisibleflag=1;
+						myx11info->myisfocusflag=1;
+					}		
+					
+					if( !myx11info->myisvisibleflag ) break;
+					
+					//liqapp_log("green shortcut"); break;
+
+						liqx11info_overlay_bring_online(myx11info);
+						if(myx11info->myoverlay)liqx11overlay_refreshdisplay(myx11info->myoverlay);
                         
                         liqx11info_regrab_focus(myx11info);
 					break;				
 		
 				case LeaveNotify:
-					liqapp_log("event.LeaveNotify");
+					if(x11_seeevent)liqapp_log("x11.event.LeaveNotify");
+					
+					if( !myx11info->myisvisibleflag ) break;
+					
+					//liqapp_log("green shortcut"); break;
 					
 					//XUnmapWindow(mydisplay,mywindow);
 	
@@ -602,20 +936,17 @@ foo:
 	
 				//############################################ focus
 				case FocusIn:					
-					liqapp_log("event.FocusIn");
+					if(x11_seeevent)liqapp_log("x11.event.FocusIn");
+					//liqapp_log("green shortcut"); break;
 					myx11info->myisfocusflag=1;
-						if((myx11info->myoverlay))
-						{
-							liqx11overlay_show(myx11info->myoverlay);
-							liqx11overlay_refreshdisplay(myx11info->myoverlay);
-							//isbusyrendering=1;
-						}
+                        liqx11info_overlay_bring_online(myx11info);
+						if(myx11info->myoverlay)liqx11overlay_refreshdisplay(myx11info->myoverlay);
                         
                         liqx11info_regrab_focus(myx11info);
 					break;				
 		
 				case FocusOut:
-					liqapp_log("event.FocusOut");
+					if(x11_seeevent)liqapp_log("x11.event.FocusOut");
 					myx11info->myisfocusflag=0;
 	
 						if((myx11info->myoverlay))
@@ -623,8 +954,8 @@ foo:
 							//liqx11overlay_hide(myx11info->myoverlay);
 							//liqx11overlay_refreshdisplay(myx11info->myoverlay);
 
-							liqx11overlay_show(myx11info->myoverlay);
-							liqx11overlay_refreshdisplay(myx11info->myoverlay);
+							//liqx11overlay_show(myx11info->myoverlay);
+							//liqx11overlay_refreshdisplay(myx11info->myoverlay);
 							//isbusyrendering=1;
 						}
 						
@@ -633,14 +964,18 @@ foo:
 				//############################################ render
 				
 				case Expose:
-					liqapp_log("event.expose count=%i         //render()",xev.xexpose.count);
+					if(x11_seeevent)liqapp_log("x11.event.expose count=%i         //render()",xev.xexpose.count);
+					//liqapp_log("green shortcut"); break;
+					
+					cover_image_blit(myx11info);
+					
 					if (xev.xexpose.count == 0)
 					{
 						
 						if(!myx11info->myoverlay)
 						{
 							myx11info->myoverlay = &myx11info->myoverlaycore;
-							liqx11overlay_init(myx11info->myoverlay, myx11info->mydisplay,myx11info->myscreen,myx11info->mywindow,myx11info->mygc);
+							liqx11overlay_init(myx11info->myoverlay, myx11info->mydisplay,myx11info->myscreen,myx11info->mywindow,myx11info->mygc,    canvas.pixelwidth,canvas.pixelheight);
 							liqx11overlay_show(myx11info->myoverlay);
 						}
 						
@@ -672,6 +1007,11 @@ foo:
 				case KeyPress:
 					//liqapp_log("keypre in  mymod=%i prev=%i",liqx11_modifier,liqx11_modifierprev);
 					{
+
+
+
+                        liqx11info_overlay_bring_online(myx11info);
+
 						// first, get the unmodified key
 						int origkeysym = XLookupKeysym((XKeyEvent*)&xev,0);
 						// find out if we need to adjust the mask
@@ -737,12 +1077,8 @@ foo:
 							}
 						}
 					}
-					//liqapp_log("keypre out mymod=%i prev=%i",liqx11_modifier,liqx11_modifierprev);
-					
-	
-					
-					
-					
+					liqapp_log("keypre out mymod=%i prev=%i",liqx11_modifier,liqx11_modifierprev);
+
 
 					
 					
@@ -767,7 +1103,34 @@ foo:
 					liqapp_log("xNextEvent keypress %i, %i",ev->key.keycode,((XKeyEvent*)&xev)->state);
 			
 			
-			
+					
+	
+						if( (ev->type == LIQEVENT_TYPE_KEY) && (ev->state==LIQEVENT_STATE_PRESS) && (ev->key.keycode==114) && (((XKeyEvent*)&xev)->state==4) )	//CTRL+r
+						{					
+					
+							//if( liqapp_pref_checkexists("forcerotation") )
+							//if(0)
+							{
+								if( atoi(liqapp_pref_getvalue("forcerotation"))==1)
+								{
+									liqapp_pref_setvalue("forcerotation","0");
+									liqapp_prefs_save();
+								}
+								else
+								{
+									liqapp_pref_setvalue("forcerotation","1");
+									liqapp_prefs_save();						
+								}
+							}
+							// now, close the overlay..
+							liqx11overlay_close(myx11info->myoverlay);
+							myx11info->myoverlay = NULL;							
+							liqx11info_overlay_bring_online(myx11info);
+							
+							// force us to reevaluate outside :)
+							ev->type = LIQEVENT_TYPE_RESIZE;
+
+						}
 			
 						// right in the core i can save a screen when i press the [Fullscreen] key
 						if( (ev->type == LIQEVENT_TYPE_KEY) && (ev->state==LIQEVENT_STATE_PRESS) && (ev->key.keycode==65475) )	//FullScreen
@@ -781,7 +1144,7 @@ foo:
 				
 				case KeyRelease:
 					
-					
+                        liqx11info_overlay_bring_online(myx11info);					
 					//liqapp_log("keyrel in  mymod=%i prev=%i",liqx11_modifier,liqx11_modifierprev);
 					
 					liqx11_modifierprev = liqx11_modifier;
@@ -858,7 +1221,8 @@ foo:
 				case 69:
 				case 76:
 				case 77:
-						//liqapp_log("event.XShmCompletionEvent");
+				case 99:
+						if(x11_seeevent)liqapp_log("x11.event.XShmCompletionEvent");
 						//isbusyrendering=0;
 						ev->type = LIQEVENT_TYPE_REFRESHED;
 						break;	
@@ -878,6 +1242,7 @@ foo:
 						
 							if (xev.type == xsp_event_base)  // RAWMOUSE
 							{
+								liqx11info_overlay_bring_online(myx11info);
 								
 								//if(!xv_canvas_inNotify) goto foo;
 								
@@ -913,20 +1278,17 @@ foo:
 						//#ifndef USE_MAEMO	
 						//		if(!myx11info->myinnotifyflag) goto foo;
 						//#endif
+								liqx11info_overlay_bring_online(myx11info);
 						
-								int xres   = ScreenOfDisplay (myx11info->mydisplay, DefaultScreen(myx11info->mydisplay))->width;
-								int yres   = ScreenOfDisplay (myx11info->mydisplay, DefaultScreen(myx11info->mydisplay))->height;
+								//int xres   = ScreenOfDisplay (myx11info->mydisplay, DefaultScreen(myx11info->mydisplay))->width;
+								//int yres   = ScreenOfDisplay (myx11info->mydisplay, DefaultScreen(myx11info->mydisplay))->height;
 							
 						
 								
 								myx11info->myispressedflag=1;
+								x11_event_mouse_coord_convert( myx11info,&xev );
 								
-                if(canvas.fullscreen)
-                {
-								xev.xmotion.x= xev.xmotion.x * canvas.pixelwidth  / xres;
-								xev.xmotion.y= xev.xmotion.y * canvas.pixelheight / yres;
-                }			
-								//liqapp_log("buttonpress  %i,%i",xev.xmotion.x,xev.xmotion.y);
+								liqapp_log("buttonpress  %i,%i",xev.xmotion.x,xev.xmotion.y);
 								//liqapp_log("ButtonPress");
 								ev->type = LIQEVENT_TYPE_MOUSE;
 								ev->mouse.state = LIQEVENT_STATE_PRESS;
@@ -938,21 +1300,21 @@ foo:
 							else if(xev.type==MotionNotify)
 							{
 								if(!myx11info->myispressedflag)goto foo;
+								
+								liqx11info_overlay_bring_online(myx11info);
+								
 						//#ifndef USE_MAEMO
 						//		if(!myx11info->myinnotifyflag) goto foo;
 						//#endif
-								int xres   = ScreenOfDisplay (myx11info->mydisplay, DefaultScreen(myx11info->mydisplay))->width;
-								int yres   = ScreenOfDisplay (myx11info->mydisplay, DefaultScreen(myx11info->mydisplay))->height;
+								//int xres   = ScreenOfDisplay (myx11info->mydisplay, DefaultScreen(myx11info->mydisplay))->width;
+								//int yres   = ScreenOfDisplay (myx11info->mydisplay, DefaultScreen(myx11info->mydisplay))->height;
 								
                                 
                 //                int qx=xev.xmotion.x;
                 //                int qy=xev.xmotion.y;
-                if(canvas.fullscreen)
-                {
-								xev.xmotion.x= xev.xmotion.x * canvas.pixelwidth  / xres;
-								xev.xmotion.y= xev.xmotion.y * canvas.pixelheight / yres;
-                }		 
-				//				liqapp_log("motion %i,%i (q %i,%i) (can %i,%i)",xev.xmotion.x,xev.xmotion.y,qx,qy,canvas.pixelwidth,canvas.pixelheight);
+								x11_event_mouse_coord_convert( myx11info,&xev );
+				
+								liqapp_log("motion %i,%i (can %i,%i)",xev.xmotion.x,xev.xmotion.y,canvas.pixelwidth,canvas.pixelheight);
 								ev->type = LIQEVENT_TYPE_MOUSE;
 								ev->mouse.state = LIQEVENT_STATE_MOVE;
 								ev->mouse.x = ((xev.xmotion.x));// *800)/canvas.pixelwidth;
@@ -964,18 +1326,16 @@ foo:
 							{
 						//#ifndef USE_MAEMO		
 						//		if(!myx11info->myinnotifyflag) goto foo;
-						//#endif		
-								int xres   = ScreenOfDisplay (myx11info->mydisplay, DefaultScreen(myx11info->mydisplay))->width;
-								int yres   = ScreenOfDisplay (myx11info->mydisplay, DefaultScreen(myx11info->mydisplay))->height;
+						//#endif
+								liqx11info_overlay_bring_online(myx11info);
+						
+								//int xres   = ScreenOfDisplay (myx11info->mydisplay, DefaultScreen(myx11info->mydisplay))->width;
+								//int yres   = ScreenOfDisplay (myx11info->mydisplay, DefaultScreen(myx11info->mydisplay))->height;
 								
 								myx11info->myispressedflag=0;
-								
-                if(canvas.fullscreen)
-                {
-								xev.xmotion.x= xev.xmotion.x * canvas.pixelwidth  / xres;
-								xev.xmotion.y= xev.xmotion.y * canvas.pixelheight / yres;
-                }		 
-								//liqapp_log("release %i,%i",xev.xmotion.x,xev.xmotion.y);
+
+								x11_event_mouse_coord_convert( myx11info,&xev );
+								liqapp_log("release %i,%i",xev.xmotion.x,xev.xmotion.y);
 						
 								//liqapp_log("ButtonRelease");
 								ev->type = LIQEVENT_TYPE_MOUSE;
@@ -992,7 +1352,7 @@ foo:
 							else
 							{
 
-
+								liqx11info_overlay_bring_online(myx11info);
 
 
 
@@ -1012,362 +1372,10 @@ foo:
 	
 	
 	return 0;
-	
-	
-	
-	
-	
-	
-	
-	
-	
-	
-	
-	
-	
-	
-	
-	
-	
-	
-	
-	
-	if(xev.type==KeyPress)
-	{
-		
-		if(!myx11info->myinnotifyflag) goto foo;
-
-
-
-		
-		ev->type = LIQEVENT_TYPE_KEY;
-		ev->key.state = LIQEVENT_STATE_PRESS;
-		ev->key.keycode = XLookupKeysym((XKeyEvent*)&xev,0); //xev.xkey.keycode;
-		ev->key.keystring[0]=0;
-		
-		keylen = XLookupString((XKeyEvent*)&xev,ev->key.keystring,16, (KeySym *) NULL, (XComposeStatus *) NULL);
-		liqapp_log("xNextEvent keypress %i",ev->key.keycode);
-
-
-
-
-			// right in the core i can save a screen when i press the [Fullscreen] key
-			if( (ev->type == LIQEVENT_TYPE_KEY) && (ev->state==LIQEVENT_STATE_PRESS) && (ev->key.keycode==65475) )	//FullScreen
-			{
-				char 		fmtnow[255];
-	 			liqapp_formatnow(fmtnow,255,"yyyymmdd_hhmmss");
-				char buf[FILENAME_MAX+1];
-				int pngerr =0;
-				
-				
-				liqimage *imgfrom=NULL;//liqcamera_getimage();
-				if(!imgfrom)
-				{
-					// camera not on, we are doing desktop
-					imgfrom = canvas.surface;
-				}
-				if(imgfrom)
-				{
-
-				/*		if(imgfrom != canvas.surface)
-						{
-							// save camera image
-							
-							snprintf(buf,FILENAME_MAX,"%s/liq.%s.%s.cam.png",app.sketchpath,fmtnow,app.username  );
-							liqapp_log("Fullscreen Pressed, saving camera  as '%s'",buf);
-							pngerr=liqimage_pagesavepng(imgdesk,buf);
-						}
-						else
-				*/
-						{
-							// save screenshot
-							snprintf(buf,FILENAME_MAX,"liq.%s.%s.scr.png",fmtnow,"lib"  );
-							liqapp_log("Fullscreen Pressed, saving canvas as '%s'",buf);
-							pngerr=liqimage_pagesavepng(imgfrom,buf);
-						}
-						
-				}
-				else
-				{
-					// no canvas
-					liqapp_log("Fullscreen Pressed, no canvas");
-				}
-			}
-	}
-	
-	
-	
-	else if(xev.type==KeyRelease)
-	{
-		
-		if(!myx11info->myinnotifyflag) goto foo;
-		
-		ev->type = LIQEVENT_TYPE_KEY;
-		ev->key.state = LIQEVENT_STATE_RELEASE;
-		ev->key.keycode =XLookupKeysym((XKeyEvent*)&xev,0); //xev.xkey.keycode;
-		ev->key.keystring[0]=0;
-		keylen = XLookupString((XKeyEvent*)&xev,ev->key.keystring,16, (KeySym *) NULL, (XComposeStatus *) NULL);
-		liqapp_log("xNextEvent keyrelease %i",ev->key.keycode);
-	}
-
-
-
-
-
-#ifdef USE_XSP
-
-	
-
-
-	else if (xev.type == xsp_event_base)  // RAWMOUSE
-	{
-		
-		//if(!xv_canvas_inNotify) goto foo;
-		
-		
-		XSPRawTouchscreenEvent xsp_event;
-		memcpy(&xsp_event, &xev,sizeof(XSPRawTouchscreenEvent));
-
-		//liqapp_log("RawMouse %i,%i,%i",xsp_event.x,xsp_event.y,xsp_event.pressure);
-		int t_x = xsp_event.x;
-		int t_y = xsp_event.y;
-
-		/* translate raw coordinates */
-		TRANSLATE_RAW_COORDS(&t_x, &t_y,canvas.pixelwidth,canvas.pixelheight);
-		if(t_x>=canvas.pixelwidth )t_x=canvas.pixelwidth -1;
-		if(t_y>=canvas.pixelheight)t_y=canvas.pixelheight-1;
-		if(t_x<0)t_x=0;
-		if(t_y<0)t_y=0;
-
-		ev->type = LIQEVENT_TYPE_MOUSE;
-		ev->mouse.state = LIQEVENT_STATE_MOVE;
-		ev->mouse.x = ((t_x));//*canvas.scalew);
-		ev->mouse.y = ((t_y));//*canvas.scaleh);
-		ev->mouse.pressure = xsp_event.pressure;
-
-	}	
-	
-#else // USE_XSP
-
-
-	else if(xev.type==ButtonPress)
-	{
-//#ifndef USE_MAEMO	
-//		if(!myx11info->myinnotifyflag) goto foo;
-//#endif
-
-		int xres   = ScreenOfDisplay (myx11info->mydisplay, DefaultScreen(myx11info->mydisplay))->width;
-		int yres   = ScreenOfDisplay (myx11info->mydisplay, DefaultScreen(myx11info->mydisplay))->height;
-	
-
-		
-        myx11info->myispressedflag=1;
-        
-        xev.xmotion.x= xev.xmotion.x * canvas.pixelwidth  / xres;
-        xev.xmotion.y= xev.xmotion.y * canvas.pixelheight / yres;
-        
-        //liqapp_log("buttonpress  %i,%i",xev.xmotion.x,xev.xmotion.y);
-		//liqapp_log("ButtonPress");
-		ev->type = LIQEVENT_TYPE_MOUSE;
-		ev->mouse.state = LIQEVENT_STATE_PRESS;
-		ev->mouse.x = ((xev.xmotion.x));
-		ev->mouse.y = ((xev.xmotion.y));
-        ev->mouse.pressure = 100;
-	}
-	else if(xev.type==MotionNotify)
-	{
-		if(!myx11info->myispressedflag)goto foo;
-//#ifndef USE_MAEMO
-//		if(!myx11info->myinnotifyflag) goto foo;
-//#endif
-		int xres   = ScreenOfDisplay (myx11info->mydisplay, DefaultScreen(myx11info->mydisplay))->width;
-		int yres   = ScreenOfDisplay (myx11info->mydisplay, DefaultScreen(myx11info->mydisplay))->height;
-		
-        xev.xmotion.x= xev.xmotion.x * canvas.pixelwidth  / xres;
-        xev.xmotion.y= xev.xmotion.y * canvas.pixelheight / yres;
- 
-        //liqapp_log("motion %i,%i",xev.xmotion.x,xev.xmotion.y);
-		ev->type = LIQEVENT_TYPE_MOUSE;
-		ev->mouse.state = LIQEVENT_STATE_MOVE;
-		ev->mouse.x = ((xev.xmotion.x));// *800)/canvas.pixelwidth;
-		ev->mouse.y = ((xev.xmotion.y));// *480)/canvas.pixelheight;
-        ev->mouse.pressure = 200;
-	}
-    else if(xev.type==ButtonRelease)
-	{
-//#ifndef USE_MAEMO		
-//		if(!myx11info->myinnotifyflag) goto foo;
-//#endif		
-		int xres   = ScreenOfDisplay (myx11info->mydisplay, DefaultScreen(myx11info->mydisplay))->width;
-		int yres   = ScreenOfDisplay (myx11info->mydisplay, DefaultScreen(myx11info->mydisplay))->height;
-		
-        myx11info->myispressedflag=0;
-        
-        xev.xmotion.x= xev.xmotion.x * canvas.pixelwidth  / xres;
-        xev.xmotion.y= xev.xmotion.y * canvas.pixelheight / yres;
- 
-        //liqapp_log("release %i,%i",xev.xmotion.x,xev.xmotion.y);
-
-		//liqapp_log("ButtonRelease");
-		ev->type = LIQEVENT_TYPE_MOUSE;
-		ev->mouse.state = LIQEVENT_STATE_RELEASE;
-		ev->mouse.x = ((xev.xmotion.x));
-		ev->mouse.y = ((xev.xmotion.y));
-        ev->mouse.pressure = 0;
-	}
-	
-	
-#endif	
-	
-	
-	
-	
-	
-	
-	
-	
-	
-	
-	
-	
-	
-	
-    
- 	else if (xev.type == 25)  // ResizeRequest
-	{
-		// obtain new dimensions and adjust our renderer to suit   
-	}
-	else if (xev.type == 65 || xev.type == 69 || xev.type == 77|| xev.type == 77)  // XShmCompletionEvent
-	{
-		//http://www.x.org/docs/Xv/video		
-		//	XvShmPutImage has a parameter:
-		//   send_event - Indicates whether or not an XShmCompletionEvent should be
-		//                sent.  If sent, the event's major_code and minor_code
-		//                fields will indicate the Xv extension's major code and
-		//                XvShmPutImage's minor code.
-		//if(myx11info->myinnotifyflag)
-		{
-		
-			ev->type = LIQEVENT_TYPE_REFRESHED;
-		}
-		//else
-		//{
-		//	// actually no point in telling the caller that we have finished refreshing :)
-		//	// lets leave him waiting for now
-		//}
-		
-	}
-	
-
-	else if (xev.type == EnterNotify) // EnterNotify
-	{
-		liqapp_log("event: EnterNotify");
-		//XRaiseWindow(dpy,window);
-		myx11info->myinnotifyflag = 1;
-	}
-	
-	
-
-	else if (xev.type == LeaveNotify) // LeaveNotify
-	{
-		liqapp_log("event: LeaveNotify");
-		//XRaiseWindow(dpy,window);
-		myx11info->myinnotifyflag = 0;
-	}
-	
-	
-	
-	
-	
-	
-
-
-	else if (xev.type == VisibilityNotify) // VisibilityNotify
-	{
-		liqapp_log("event: VisibilityNotify %i",xev.xvisibility.state);
-		//XRaiseWindow(dpy,window);
-	}
-	
-	
-	
-	
-	else if (xev.type == Expose) // expose
-	{
-		//XRaiseWindow(dpy,window);
-		//liqcanvas_refreshdisplay();
-		if(xev.xexpose.count>0) goto foo;
-
-
-		{
-			liqapp_log("event: Expose");
-			ev->type = LIQEVENT_TYPE_EXPOSE;
-		}
-		
-	}
-	
-	else if (xev.type == MappingNotify) // mapping notify
-	{
-		liqapp_log("event: MappingNotify");
-		//XRaiseWindow(dpy,window);
-	}
-	
-	else if (xev.type == MapNotify) // map notify
-	{
-		liqapp_log("event: MapNotify");
-		//XRaiseWindow(dpy,window);
-	}
-	
-	else if (xev.type == UnmapNotify) // unmap notify
-	{
-		liqapp_log("event: UnmapNotify");
-		//XRaiseWindow(dpy,window);
-	}
-	else if (xev.type == ClientMessage) // client message
-	{
-		liqapp_log("event: ClientMessage");
-		//XRaiseWindow(dpy,window);
-	}	
-	
-	else if (xev.type == 77) // ..
-	{
-		//liqapp_log("event: ...");
-		//XRaiseWindow(dpy,window);
-	}	
-	
-	else if (xev.type == 22) // ..
-	{
-		//liqapp_log("event: ...");
-		//XRaiseWindow(dpy,window);
-	}	
-
-	else if (xev.type == 94)// random events which dont occur in maemo..
-	{
-	
-	}
-	else if (xev.type == 6)// random events which dont occur in maemo.. came at a mouse move..
-	{
-	
-	}
-	else
-	{
-
-	// x11 event definitions: http://www.openmash.org/lxr/source/xlib/X11/X.h?c=tk8.3#L99
-	
-	//		ev->type = LIQEVENT_TYPE_UNKNOWN;
-			liqapp_log("Unknown event %i, let me know what happened at liquid@gmail.com",xev.type);
-			//exit(-1);
-	}
-
-	return 0;
 
 }
 
-
-
-
-
-
-
-
-
+#ifdef __cplusplus
+}
+#endif
 
